import unittest
import tableauserverclient as TSC
import tableauserverclient.server.request_factory as TSC_RF
from tableauserverclient.helpers.strings import redact_xml
import pytest
import sys


class WorkbookRequestTests(unittest.TestCase):
    def test_embedded_extract_req(self):
        include_all = True
        embedded_datasources = None
        xml_result = TSC_RF.RequestFactory.Workbook.embedded_extract_req(include_all, embedded_datasources)

    def test_generate_xml(self):
        workbook_item: TSC.WorkbookItem = TSC.WorkbookItem("name", "project_id")
        TSC_RF.RequestFactory.Workbook._generate_xml(workbook_item)

    def test_generate_xml_invalid_connection(self):
        workbook_item: TSC.WorkbookItem = TSC.WorkbookItem("name", "project_id")
        conn = TSC.ConnectionItem()
        with self.assertRaises(ValueError):
            request = TSC_RF.RequestFactory.Workbook._generate_xml(workbook_item, connections=[conn])

    def test_generate_xml_invalid_connection_credentials(self):
        workbook_item: TSC.WorkbookItem = TSC.WorkbookItem("name", "project_id")
        conn = TSC.ConnectionItem()
        conn.server_address = "address"
        creds = TSC.ConnectionCredentials("username", "password")
        creds.name = None
        conn.connection_credentials = creds
        with self.assertRaises(ValueError):
            request = TSC_RF.RequestFactory.Workbook._generate_xml(workbook_item, connections=[conn])

    def test_generate_xml_valid_connection_credentials(self):
        workbook_item: TSC.WorkbookItem = TSC.WorkbookItem("name", "project_id")
        conn = TSC.ConnectionItem()
        conn.server_address = "address"
        creds = TSC.ConnectionCredentials("username", "DELETEME")
        conn.connection_credentials = creds
        request = TSC_RF.RequestFactory.Workbook._generate_xml(workbook_item, connections=[conn])
        assert request.find(b"DELETEME") > 0

    def test_redact_passwords_in_xml(self):
        if sys.version_info < (3, 7):
            pytest.skip("Redaction is only implemented for 3.7+.")
        workbook_item: TSC.WorkbookItem = TSC.WorkbookItem("name", "project_id")
        conn = TSC.ConnectionItem()
        conn.server_address = "address"
        creds = TSC.ConnectionCredentials("username", "DELETEME")
        conn.connection_credentials = creds
        request = TSC_RF.RequestFactory.Workbook._generate_xml(workbook_item, connections=[conn])
        redacted = redact_xml(request)
        assert request.find(b"DELETEME") > 0, request
        assert redacted.find(b"DELETEME") == -1, redacted



####
# This script demonstrates how to query for permissions using TSC
# To run the script, you must have installed Python 3.7 or later.
#
# Example usage: 'python query_permissions.py -s https://10ax.online.tableau.com --site
#       devSite123 -u tabby@tableau.com workbook b4065286-80f0-11ea-af1b-cb7191f48e45'
####

import argparse
import logging

import tableauserverclient as TSC


def main():
    parser = argparse.ArgumentParser(description="Query permissions of a given resource.")
    # Common options; please keep those in sync across all samples
    parser.add_argument("--server", "-s", required=True, help="server address")
    parser.add_argument("--site", "-S", help="site name")
    parser.add_argument(
        "--token-name", "-p", required=True, help="name of the personal access token used to sign into the server"
    )
    parser.add_argument(
        "--token-value", "-v", required=True, help="value of the personal access token used to sign into the server"
    )
    parser.add_argument(
        "--logging-level",
        "-l",
        choices=["debug", "info", "error"],
        default="error",
        help="desired logging level (set to error by default)",
    )
    # Options specific to this sample
    parser.add_argument("resource_type", choices=["workbook", "datasource", "flow", "table", "database"])
    parser.add_argument("resource_id")

    args = parser.parse_args()

    # Set logging level based on user input, or error by default
    logging_level = getattr(logging, args.logging_level.upper())
    logging.basicConfig(level=logging_level)

    # Sign in
    tableau_auth = TSC.PersonalAccessTokenAuth(args.token_name, args.token_value, site_id=args.site)
    server = TSC.Server(args.server, use_server_version=True)
    with server.auth.sign_in(tableau_auth):
        # Mapping to grab the handler for the user-inputted resource type
        endpoint = {
            "workbook": server.workbooks,
            "datasource": server.datasources,
            "flow": server.flows,
            "table": server.tables,
            "database": server.databases,
        }.get(args.resource_type)

        # Get the resource by its ID
        resource = endpoint.get_by_id(args.resource_id)

        # Populate permissions for the resource
        endpoint.populate_permissions(resource)
        permissions = resource.permissions

        # Print result
        print(
            "\n{0} permission rule(s) found for {1} {2}.".format(len(permissions), args.resource_type, args.resource_id)
        )

        for permission in permissions:
            grantee = permission.grantee
            capabilities = permission.capabilities
            print("\nCapabilities for {0} {1}:".format(grantee.tag_name, grantee.id))

            for capability in capabilities:
                print("\t{0} - {1}".format(capability, capabilities[capability]))


if __name__ == "__main__":
    main()



from .request_options import RequestOptions


class Filter(object):
    def __init__(self, field, operator, value):
        self.field = field
        self.operator = operator
        self._value = None
        self.value = value

    def __str__(self):
        value_string = str(self._value)
        if isinstance(self._value, list):
            value_string = value_string.replace(" ", "").replace("'", "")
        return "{0}:{1}:{2}".format(self.field, self.operator, value_string)

    @property
    def value(self):
        return self._value

    @value.setter
    def value(self, filter_value):
        if isinstance(filter_value, list) and self.operator != RequestOptions.Operator.In:
            error = "Filter values can only be a list if the operator is 'in'."
            raise ValueError(error)
        else:
            self._value = filter_value



import os.path
import unittest

import requests_mock

import tableauserverclient as TSC

TEST_ASSET_DIR = os.path.join(os.path.dirname(__file__), "assets")

SERVER_INFO_GET_XML = os.path.join(TEST_ASSET_DIR, "server_info_get.xml")
SERVER_INFO_25_XML = os.path.join(TEST_ASSET_DIR, "server_info_25.xml")
SERVER_INFO_404 = os.path.join(TEST_ASSET_DIR, "server_info_404.xml")
SERVER_INFO_AUTH_INFO_XML = os.path.join(TEST_ASSET_DIR, "server_info_auth_info.xml")


class ServerInfoTests(unittest.TestCase):
    def setUp(self):
        self.server = TSC.Server("http://test", False)
        self.baseurl = self.server.server_info.baseurl
        self.server.version = "2.4"

    def test_server_info_get(self):
        with open(SERVER_INFO_GET_XML, "rb") as f:
            response_xml = f.read().decode("utf-8")
        with requests_mock.mock() as m:
            m.get(self.server.server_info.baseurl, text=response_xml)
            actual = self.server.server_info.get()

            self.assertEqual("10.1.0", actual.product_version)
            self.assertEqual("10100.16.1024.2100", actual.build_number)
            self.assertEqual("3.10", actual.rest_api_version)

    def test_server_info_use_highest_version_downgrades(self):
        with open(SERVER_INFO_AUTH_INFO_XML, "rb") as f:
            # This is the auth.xml endpoint present back to 9.0 Servers
            auth_response_xml = f.read().decode("utf-8")
        with open(SERVER_INFO_404, "rb") as f:
            # 10.1 serverInfo response
            si_response_xml = f.read().decode("utf-8")
        with requests_mock.mock() as m:
            # Return a 404 for serverInfo so we can pretend this is an old Server
            m.get(self.server.server_address + "/api/2.4/serverInfo", text=si_response_xml, status_code=404)
            m.get(self.server.server_address + "/auth?format=xml", text=auth_response_xml)
            self.server.use_server_version()
            # does server-version[9.2] lookup in PRODUCT_TO_REST_VERSION
            self.assertEqual(self.server.version, "2.2")

    def test_server_info_use_highest_version_upgrades(self):
        with open(SERVER_INFO_GET_XML, "rb") as f:
            si_response_xml = f.read().decode("utf-8")
        with requests_mock.mock() as m:
            m.get(self.server.server_address + "/api/2.8/serverInfo", text=si_response_xml)
            # Pretend we're old
            self.server.version = "2.8"
            self.server.use_server_version()
            # Did we upgrade to 3.10?
            self.assertEqual(self.server.version, "3.10")

    def test_server_use_server_version_flag(self):
        with open(SERVER_INFO_25_XML, "rb") as f:
            si_response_xml = f.read().decode("utf-8")
        with requests_mock.mock() as m:
            m.get("http://test/api/2.4/serverInfo", text=si_response_xml)
            server = TSC.Server("http://test", use_server_version=True)
            self.assertEqual(server.version, "2.5")



import json
import logging

from .endpoint import Endpoint, api
from .exceptions import GraphQLError, InvalidGraphQLQuery

logger = logging.getLogger("tableau.endpoint.metadata")


def is_valid_paged_query(parsed_query):
    """Check that the required $first and $afterToken variables are present in the query.
    Also check that we are asking for the pageInfo object, so we get the endCursor. There
    is no way to do this relilably without writing a GraphQL parser, so simply check that
    that the string contains 'hasNextPage' and 'endCursor'"""
    return (
        all(k in parsed_query["variables"] for k in ("first", "afterToken"))
        and "hasNextPage" in parsed_query["query"]
        and "endCursor" in parsed_query["query"]
    )


def extract_values(obj, key):
    """Pull all values of specified key from nested JSON.
    Taken from: https://hackersandslackers.com/extract-data-from-complex-json-python/"""
    arr = []

    def extract(obj, arr, key):
        """Recursively search for values of key in JSON tree."""
        if isinstance(obj, dict):
            for k, v in obj.items():
                if isinstance(v, (dict, list)):
                    extract(v, arr, key)
                elif k == key:
                    arr.append(v)
        elif isinstance(obj, list):
            for item in obj:
                extract(item, arr, key)
        return arr

    results = extract(obj, arr, key)
    return results


def get_page_info(result):
    next_page = extract_values(result, "hasNextPage").pop()
    cursor = extract_values(result, "endCursor").pop()
    return next_page, cursor


class Metadata(Endpoint):
    @property
    def baseurl(self):
        return "{0}/api/metadata/graphql".format(self.parent_srv.server_address)

    @property
    def control_baseurl(self):
        return "{0}/api/metadata/v1/control".format(self.parent_srv.server_address)

    @api("3.5")
    def query(self, query, variables=None, abort_on_error=False, parameters=None):
        logger.info("Querying Metadata API")

        url = self.baseurl

        try:
            graphql_query = json.dumps({"query": query, "variables": variables})
        except Exception as e:
            raise InvalidGraphQLQuery("Must provide a string")

        # Setting content type because post_reuqest defaults to text/xml
        server_response = self.post_request(url, graphql_query, content_type="application/json", parameters=parameters)
        results = server_response.json()

        if abort_on_error and results.get("errors", None):
            raise GraphQLError(results["errors"])

        return results

    @api("3.9")
    def backfill_status(self):
        url = self.control_baseurl + "/backfill/status"
        response = self.get_request(url)
        return response.json()

    @api("3.9")
    def eventing_status(self):
        url = self.control_baseurl + "/eventing/status"
        response = self.get_request(url)
        return response.json()

    @api("3.5")
    def paginated_query(self, query, variables=None, abort_on_error=False):
        logger.info("Querying Metadata API using a Paged Query")
        url = self.baseurl

        if variables is None:
            # default paramaters
            variables = {"first": 100, "afterToken": None}
        elif ("first" in variables) and ("afterToken" not in variables):
            # they passed a page size but not a token, probably because they're starting at `null` token
            variables.update({"afterToken": None})

        graphql_query = json.dumps({"query": query, "variables": variables})
        parsed_query = json.loads(graphql_query)

        if not is_valid_paged_query(parsed_query):
            raise InvalidGraphQLQuery(
                "Paged queries must have a `$first` and `$afterToken` variables as well as "
                "a pageInfo object with `endCursor` and `hasNextPage`"
            )

        results_dict = {"pages": []}
        paginated_results = results_dict["pages"]

        # get first page
        server_response = self.post_request(url, graphql_query, content_type="application/json")
        results = server_response.json()

        if abort_on_error and results.get("errors", None):
            raise GraphQLError(results["errors"])

        paginated_results.append(results)

        # repeat
        has_another_page, cursor = get_page_info(results)

        while has_another_page:
            # Update the page
            variables.update({"afterToken": cursor})
            # make the call
            logger.debug("Calling Token: " + cursor)
            graphql_query = json.dumps({"query": query, "variables": variables})
            server_response = self.post_request(url, graphql_query, content_type="application/json")
            results = server_response.json()
            # verify response
            if abort_on_error and results.get("errors", None):
                raise GraphQLError(results["errors"])
            # save results and repeat
            paginated_results.append(results)
            has_another_page, cursor = get_page_info(results)

        logger.info("Sucessfully got all results for paged query")
        return results_dict



import os
import unittest

import requests_mock

from tableauserverclient.server import Server
from ._utils import asset

TEST_ASSET_DIR = os.path.join(os.path.dirname(__file__), "assets")
FILEUPLOAD_INITIALIZE = os.path.join(TEST_ASSET_DIR, "fileupload_initialize.xml")
FILEUPLOAD_APPEND = os.path.join(TEST_ASSET_DIR, "fileupload_append.xml")


class FileuploadsTests(unittest.TestCase):
    def setUp(self):
        self.server = Server("http://test", False)

        # Fake sign in
        self.server._site_id = "dad65087-b08b-4603-af4e-2887b8aafc67"
        self.server._auth_token = "j80k54ll2lfMZ0tv97mlPvvSCRyD0DOM"

        self.baseurl = "{}/sites/{}/fileUploads".format(self.server.baseurl, self.server.site_id)

    def test_read_chunks_file_path(self):
        file_path = asset("SampleWB.twbx")
        chunks = self.server.fileuploads._read_chunks(file_path)
        for chunk in chunks:
            self.assertIsNotNone(chunk)

    def test_read_chunks_file_object(self):
        with open(asset("SampleWB.twbx"), "rb") as f:
            chunks = self.server.fileuploads._read_chunks(f)
            for chunk in chunks:
                self.assertIsNotNone(chunk)

    def test_upload_chunks_file_path(self):
        file_path = asset("SampleWB.twbx")
        upload_id = "7720:170fe6b1c1c7422dadff20f944d58a52-1:0"

        with open(FILEUPLOAD_INITIALIZE, "rb") as f:
            initialize_response_xml = f.read().decode("utf-8")
        with open(FILEUPLOAD_APPEND, "rb") as f:
            append_response_xml = f.read().decode("utf-8")
        with requests_mock.mock() as m:
            m.post(self.baseurl, text=initialize_response_xml)
            m.put(self.baseurl + "/" + upload_id, text=append_response_xml)
            actual = self.server.fileuploads.upload(file_path)

        self.assertEqual(upload_id, actual)

    def test_upload_chunks_file_object(self):
        upload_id = "7720:170fe6b1c1c7422dadff20f944d58a52-1:0"

        with open(asset("SampleWB.twbx"), "rb") as file_content:
            with open(FILEUPLOAD_INITIALIZE, "rb") as f:
                initialize_response_xml = f.read().decode("utf-8")
            with open(FILEUPLOAD_APPEND, "rb") as f:
                append_response_xml = f.read().decode("utf-8")
            with requests_mock.mock() as m:
                m.post(self.baseurl, text=initialize_response_xml)
                m.put(self.baseurl + "/" + upload_id, text=append_response_xml)
                actual = self.server.fileuploads.upload(file_content)

        self.assertEqual(upload_id, actual)



from pathlib import Path
import unittest

import tableauserverclient as TSC

import requests_mock

ASSETS = Path(__file__).parent / "assets"


class TestEndpoint(unittest.TestCase):
    def setUp(self) -> None:
        self.server = TSC.Server("http://test/", use_server_version=False)

        # Fake signin
        self.server._site_id = "dad65087-b08b-4603-af4e-2887b8aafc67"
        self.server._auth_token = "j80k54ll2lfMZ0tv97mlPvvSCRyD0DOM"

        return super().setUp()

    def test_get_request_stream(self) -> None:
        url = "http://test/"
        endpoint = TSC.server.Endpoint(self.server)
        with requests_mock.mock() as m:
            m.get(url, headers={"Content-Type": "application/octet-stream"})
            response = endpoint.get_request(url, parameters={"stream": True})

            self.assertFalse(response._content_consumed)

    def test_binary_log_truncated(self):
        class FakeResponse(object):
            headers = {"Content-Type": "application/octet-stream"}
            content = b"\x1337" * 1000
            status_code = 200

        endpoint = TSC.server.Endpoint(self.server)
        server_response = FakeResponse()
        log = endpoint.log_response_safely(server_response)
        self.assertTrue(log.find("[Truncated File Contents]") > 0, log)

    def test_set_user_agent_from_options_headers(self):
        params = {"User-Agent": "1", "headers": {"User-Agent": "2"}}
        result = TSC.server.Endpoint.set_user_agent(params)
        # it should use the value under 'headers' if more than one is given
        print(result)
        print(result["headers"]["User-Agent"])
        self.assertTrue(result["headers"]["User-Agent"] == "2")

    def test_set_user_agent_from_options(self):
        params = {"headers": {"User-Agent": "2"}}
        result = TSC.server.Endpoint.set_user_agent(params)
        self.assertTrue(result["headers"]["User-Agent"] == "2")

    def test_set_user_agent_when_blank(self):
        params = {"headers": {}}
        result = TSC.server.Endpoint.set_user_agent(params)
        self.assertTrue(result["headers"]["User-Agent"].startswith("Tableau Server Client"))



import os
import unittest

import requests_mock

import tableauserverclient as TSC
from tableauserverclient.server import RequestFactory, WebhookItem
from ._utils import asset

TEST_ASSET_DIR = os.path.join(os.path.dirname(__file__), "assets")

GET_XML = asset("webhook_get.xml")
CREATE_XML = asset("webhook_create.xml")
CREATE_REQUEST_XML = asset("webhook_create_request.xml")


class WebhookTests(unittest.TestCase):
    def setUp(self) -> None:
        self.server = TSC.Server("http://test", False)
        self.server.version = "3.6"

        # Fake signin
        self.server._site_id = "dad65087-b08b-4603-af4e-2887b8aafc67"
        self.server._auth_token = "j80k54ll2lfMZ0tv97mlPvvSCRyD0DOM"

        self.baseurl = self.server.webhooks.baseurl

    def test_get(self) -> None:
        with open(GET_XML, "rb") as f:
            response_xml = f.read().decode("utf-8")
        with requests_mock.mock() as m:
            m.get(self.baseurl, text=response_xml)
            webhooks, _ = self.server.webhooks.get()
            self.assertEqual(len(webhooks), 1)
            webhook = webhooks[0]

            self.assertEqual(webhook.url, "url")
            self.assertEqual(webhook.event, "datasource-created")
            self.assertEqual(webhook.owner_id, "webhook_owner_luid")
            self.assertEqual(webhook.name, "webhook-name")
            self.assertEqual(webhook.id, "webhook-id")

    def test_get_before_signin(self) -> None:
        self.server._auth_token = None
        self.assertRaises(TSC.NotSignedInError, self.server.webhooks.get)

    def test_delete(self) -> None:
        with requests_mock.mock() as m:
            m.delete(self.baseurl + "/ee8c6e70-43b6-11e6-af4f-f7b0d8e20760", status_code=204)
            self.server.webhooks.delete("ee8c6e70-43b6-11e6-af4f-f7b0d8e20760")

    def test_delete_missing_id(self) -> None:
        self.assertRaises(ValueError, self.server.webhooks.delete, "")

    def test_test(self) -> None:
        with requests_mock.mock() as m:
            m.get(self.baseurl + "/ee8c6e70-43b6-11e6-af4f-f7b0d8e20760/test", status_code=200)
            self.server.webhooks.test("ee8c6e70-43b6-11e6-af4f-f7b0d8e20760")

    def test_create(self) -> None:
        with open(CREATE_XML, "rb") as f:
            response_xml = f.read().decode("utf-8")
        with requests_mock.mock() as m:
            m.post(self.baseurl, text=response_xml)
            webhook_model = TSC.WebhookItem()
            webhook_model.name = "Test Webhook"
            webhook_model.url = "https://ifttt.com/maker-url"
            webhook_model.event = "datasource-created"

            new_webhook = self.server.webhooks.create(webhook_model)

            self.assertNotEqual(new_webhook.id, None)

    def test_request_factory(self):
        with open(CREATE_REQUEST_XML, "rb") as f:
            webhook_request_expected = f.read().decode("utf-8")

        webhook_item = WebhookItem()
        webhook_item._set_values("webhook-id", "webhook-name", "url", "api-event-name", None)
        webhook_request_actual = "{}\n".format(RequestFactory.Webhook.create_req(webhook_item).decode("utf-8"))
        self.maxDiff = None
        # windows does /r/n for linebreaks, remove the extra char if it is there
        self.assertEqual(webhook_request_expected.replace("\r", ""), webhook_request_actual)



####
# This script demonstrates how to use the Tableau Server Client
# to interact with workbooks. It explores the different
# functions that the Server API supports on workbooks.
#
# With no flags set, this sample will query all workbooks,
# pick one workbook and populate its connections/views, and update
# the workbook. Adding flags will demonstrate the specific feature
# on top of the general operations.
####

import argparse
import logging
import os.path

import tableauserverclient as TSC


def main():
    parser = argparse.ArgumentParser(description="Explore workbook functions supported by the Server API.")
    # Common options; please keep those in sync across all samples
    parser.add_argument("--server", "-s", required=True, help="server address")
    parser.add_argument("--site", "-S", help="site name")
    parser.add_argument(
        "--token-name", "-p", required=True, help="name of the personal access token used to sign into the server"
    )
    parser.add_argument(
        "--token-value", "-v", required=True, help="value of the personal access token used to sign into the server"
    )
    parser.add_argument(
        "--logging-level",
        "-l",
        choices=["debug", "info", "error"],
        default="error",
        help="desired logging level (set to error by default)",
    )
    # Options specific to this sample
    parser.add_argument("--publish", metavar="FILEPATH", help="path to workbook to publish")
    parser.add_argument("--download", metavar="FILEPATH", help="path to save downloaded workbook")
    parser.add_argument(
        "--preview-image", "-i", metavar="FILENAME", help="filename (a .png file) to save the preview image"
    )

    args = parser.parse_args()

    # Set logging level based on user input, or error by default
    logging_level = getattr(logging, args.logging_level.upper())
    logging.basicConfig(level=logging_level)

    # SIGN IN
    tableau_auth = TSC.PersonalAccessTokenAuth(args.token_name, args.token_value, site_id=args.site)
    server = TSC.Server(args.server, use_server_version=True)
    with server.auth.sign_in(tableau_auth):
        # Publish workbook if publish flag is set (-publish, -p)
        overwrite_true = TSC.Server.PublishMode.Overwrite
        if args.publish:
            all_projects, pagination_item = server.projects.get()
            default_project = next((project for project in all_projects if project.is_default()), None)

            if default_project is not None:
                new_workbook = TSC.WorkbookItem(default_project.id)
                new_workbook = server.workbooks.publish(new_workbook, args.publish, overwrite_true)
                print("Workbook published. ID: {}".format(new_workbook.id))
            else:
                print("Publish failed. Could not find the default project.")

        # Gets all workbook items
        all_workbooks, pagination_item = server.workbooks.get()
        print("\nThere are {} workbooks on site: ".format(pagination_item.total_available))
        print([workbook.name for workbook in all_workbooks])

        if all_workbooks:
            # Pick one workbook from the list
            sample_workbook = all_workbooks[0]
            sample_workbook.name = "Name me something cooler"
            sample_workbook.description = "That doesn't work"
            updated: TSC.WorkbookItem = server.workbooks.update(sample_workbook)
            print(updated.name, updated.description)

            # Populate views
            server.workbooks.populate_views(sample_workbook)
            print("\nName of views in {}: ".format(sample_workbook.name))
            print([view.name for view in sample_workbook.views])

            # Populate connections
            server.workbooks.populate_connections(sample_workbook)
            print("\nConnections for {}: ".format(sample_workbook.name))
            print(
                [
                    "{0}({1})".format(connection.id, connection.datasource_name)
                    for connection in sample_workbook.connections
                ]
            )

            # Update tags and show_tabs flag
            original_tag_set = set(sample_workbook.tags)
            sample_workbook.tags.update("a", "b", "c", "d")
            sample_workbook.show_tabs = True
            server.workbooks.update(sample_workbook)
            print("\nWorkbook's old tag set: {}".format(original_tag_set))
            print("Workbook's new tag set: {}".format(sample_workbook.tags))
            print("Workbook tabbed: {}".format(sample_workbook.show_tabs))

            # Delete all tags that were added by setting tags to original
            sample_workbook.tags = original_tag_set
            server.workbooks.update(sample_workbook)

            # Add tag to just one view
            sample_view = sample_workbook.views[0]
            original_tag_set = set(sample_view.tags)
            sample_view.tags.add("view_tag")
            server.views.update(sample_view)
            print("\nView's old tag set: {}".format(original_tag_set))
            print("View's new tag set: {}".format(sample_view.tags))

            # Delete tag from just one view
            sample_view.tags = original_tag_set
            server.views.update(sample_view)

            if args.download:
                # Download
                path = server.workbooks.download(sample_workbook.id, args.download)
                print("\nDownloaded workbook to {}".format(path))

            if args.preview_image:
                # Populate workbook preview image
                server.workbooks.populate_preview_image(sample_workbook)
                with open(args.preview_image, "wb") as f:
                    f.write(sample_workbook.preview_image)
                print("\nDownloaded preview image of workbook to {}".format(os.path.abspath(args.preview_image)))

            # get custom views
            cvs, _ = server.custom_views.get()
            for c in cvs:
                print(c)

            # for the last custom view in the list

            # update the name
            # note that this will fail if the name is already changed to this value
            changed: TSC.CustomViewItem(id=c.id, name="I was updated by tsc")
            verified_change = server.custom_views.update(changed)
            print(verified_change)

            # export as image. Filters etc could be added here as usual
            server.custom_views.populate_image(c)
            filename = c.id + "-image-export.png"
            with open(filename, "wb") as f:
                f.write(c.image)
            print("saved to " + filename)

            if args.delete:
                print("deleting {}".format(c.id))
                unlucky = TSC.CustomViewItem(c.id)
                server.custom_views.delete(unlucky.id)


if __name__ == "__main__":
    main()



####
# This script demonstrates how to use the Tableau Server Client
# to interact with sites.
####

import argparse
import logging
import os.path
import sys

import tableauserverclient as TSC


def main():
    parser = argparse.ArgumentParser(description="Explore site updates by the Server API.")
    # Common options; please keep those in sync across all samples
    parser.add_argument("--server", "-s", required=True, help="server address")
    parser.add_argument("--site", "-S", help="site name")
    parser.add_argument(
        "--token-name", "-p", required=True, help="name of the personal access token used to sign into the server"
    )
    parser.add_argument(
        "--token-value", "-v", required=True, help="value of the personal access token used to sign into the server"
    )
    parser.add_argument(
        "--logging-level",
        "-l",
        choices=["debug", "info", "error"],
        default="error",
        help="desired logging level (set to error by default)",
    )

    parser.add_argument("--delete")
    parser.add_argument("--create")
    parser.add_argument("--url")
    parser.add_argument("--new_site_name")
    parser.add_argument("--user_quota")
    parser.add_argument("--storage_quota")
    parser.add_argument("--status")

    args = parser.parse_args()

    # Set logging level based on user input, or error by default
    logging_level = getattr(logging, args.logging_level.upper())
    logging.basicConfig(level=logging_level)

    # SIGN IN
    tableau_auth = TSC.PersonalAccessTokenAuth(args.token_name, args.token_value, site_id=args.site)
    server = TSC.Server(args.server, use_server_version=True)
    new_site = None
    with server.auth.sign_in(tableau_auth):
        current_site = server.sites.get_by_id(server.site_id)

        if args.delete:
            print("You can only delete the site you are currently in")
            print("Delete site `{}`?".format(current_site.name))
            # server.sites.delete(server.site_id)

        elif args.create:
            new_site = TSC.SiteItem(args.create, args.url or args.create)
            site_item = server.sites.create(new_site)
            print(site_item)
            # to do anything further with the site, you need to log into it
            # if a PAT is required, that means going to the UI to create one

        else:
            new_site = current_site
            print(current_site, "current user quota:", current_site.user_quota)
            print("Remember, you can only update the site you are currently in")
            if args.url:
                new_site.content_url = args.url
            if args.user_quota:
                new_site.user_quota = args.user_quota
            try:
                updated_site = server.sites.update(new_site)
                print(updated_site, "new user quota:", updated_site.user_quota)
            except TSC.ServerResponseError as e:
                print(e)


if __name__ == "__main__":
    main()



import logging

from .endpoint import Endpoint, api
from tableauserverclient.server import RequestFactory
from tableauserverclient.models import FavoriteItem

from typing import Optional, TYPE_CHECKING

if TYPE_CHECKING:
    from ...models import DatasourceItem, FlowItem, ProjectItem, UserItem, ViewItem, WorkbookItem
    from ..request_options import RequestOptions

logger = logging.getLogger("tableau.endpoint.favorites")


class Favorites(Endpoint):
    @property
    def baseurl(self) -> str:
        return "{0}/sites/{1}/favorites".format(self.parent_srv.baseurl, self.parent_srv.site_id)

    # Gets all favorites
    @api(version="2.5")
    def get(self, user_item: "UserItem", req_options: Optional["RequestOptions"] = None) -> None:
        logger.info("Querying all favorites for user {0}".format(user_item.name))
        url = "{0}/{1}".format(self.baseurl, user_item.id)
        server_response = self.get_request(url, req_options)

        user_item._favorites = FavoriteItem.from_response(server_response.content, self.parent_srv.namespace)

    @api(version="2.0")
    def add_favorite_workbook(self, user_item: "UserItem", workbook_item: "WorkbookItem") -> None:
        url = "{0}/{1}".format(self.baseurl, user_item.id)
        add_req = RequestFactory.Favorite.add_workbook_req(workbook_item.id, workbook_item.name)
        server_response = self.put_request(url, add_req)
        logger.info("Favorited {0} for user (ID: {1})".format(workbook_item.name, user_item.id))

    @api(version="2.0")
    def add_favorite_view(self, user_item: "UserItem", view_item: "ViewItem") -> None:
        url = "{0}/{1}".format(self.baseurl, user_item.id)
        add_req = RequestFactory.Favorite.add_view_req(view_item.id, view_item.name)
        server_response = self.put_request(url, add_req)
        logger.info("Favorited {0} for user (ID: {1})".format(view_item.name, user_item.id))

    @api(version="2.3")
    def add_favorite_datasource(self, user_item: "UserItem", datasource_item: "DatasourceItem") -> None:
        url = "{0}/{1}".format(self.baseurl, user_item.id)
        add_req = RequestFactory.Favorite.add_datasource_req(datasource_item.id, datasource_item.name)
        server_response = self.put_request(url, add_req)
        logger.info("Favorited {0} for user (ID: {1})".format(datasource_item.name, user_item.id))

    @api(version="3.1")
    def add_favorite_project(self, user_item: "UserItem", project_item: "ProjectItem") -> None:
        url = "{0}/{1}".format(self.baseurl, user_item.id)
        add_req = RequestFactory.Favorite.add_project_req(project_item.id, project_item.name)
        server_response = self.put_request(url, add_req)
        logger.info("Favorited {0} for user (ID: {1})".format(project_item.name, user_item.id))

    @api(version="3.3")
    def add_favorite_flow(self, user_item: "UserItem", flow_item: "FlowItem") -> None:
        url = "{0}/{1}".format(self.baseurl, user_item.id)
        add_req = RequestFactory.Favorite.add_flow_req(flow_item.id, flow_item.name)
        server_response = self.put_request(url, add_req)
        logger.info("Favorited {0} for user (ID: {1})".format(flow_item.name, user_item.id))

    @api(version="2.0")
    def delete_favorite_workbook(self, user_item: "UserItem", workbook_item: "WorkbookItem") -> None:
        url = "{0}/{1}/workbooks/{2}".format(self.baseurl, user_item.id, workbook_item.id)
        logger.info("Removing favorite {0} for user (ID: {1})".format(workbook_item.id, user_item.id))
        self.delete_request(url)

    @api(version="2.0")
    def delete_favorite_view(self, user_item: "UserItem", view_item: "ViewItem") -> None:
        url = "{0}/{1}/views/{2}".format(self.baseurl, user_item.id, view_item.id)
        logger.info("Removing favorite {0} for user (ID: {1})".format(view_item.id, user_item.id))
        self.delete_request(url)

    @api(version="2.3")
    def delete_favorite_datasource(self, user_item: "UserItem", datasource_item: "DatasourceItem") -> None:
        url = "{0}/{1}/datasources/{2}".format(self.baseurl, user_item.id, datasource_item.id)
        logger.info("Removing favorite {0} for user (ID: {1})".format(datasource_item.id, user_item.id))
        self.delete_request(url)

    @api(version="3.1")
    def delete_favorite_project(self, user_item: "UserItem", project_item: "ProjectItem") -> None:
        url = "{0}/{1}/projects/{2}".format(self.baseurl, user_item.id, project_item.id)
        logger.info("Removing favorite {0} for user (ID: {1})".format(project_item.id, user_item.id))
        self.delete_request(url)

    @api(version="3.3")
    def delete_favorite_flow(self, user_item: "UserItem", flow_item: "FlowItem") -> None:
        url = "{0}/{1}/projects/{2}".format(self.baseurl, user_item.id, flow_item.id)
        logger.info("Removing favorite {0} for user (ID: {1})".format(flow_item.id, user_item.id))
        self.delete_request(url)



import os
import unittest

import requests_mock

import tableauserverclient as TSC
from tableauserverclient import GroupItem
from ._utils import read_xml_asset, asset

TEST_ASSET_DIR = os.path.join(os.path.dirname(__file__), "assets")

GET_XML = asset("project_get.xml")
UPDATE_XML = asset("project_update.xml")
SET_CONTENT_PERMISSIONS_XML = asset("project_content_permission.xml")
CREATE_XML = asset("project_create.xml")
POPULATE_PERMISSIONS_XML = "project_populate_permissions.xml"
POPULATE_WORKBOOK_DEFAULT_PERMISSIONS_XML = "project_populate_workbook_default_permissions.xml"
UPDATE_DATASOURCE_DEFAULT_PERMISSIONS_XML = "project_update_datasource_default_permissions.xml"


class ProjectTests(unittest.TestCase):
    def setUp(self) -> None:
        self.server = TSC.Server("http://test", False)

        # Fake signin
        self.server._site_id = "dad65087-b08b-4603-af4e-2887b8aafc67"
        self.server._auth_token = "j80k54ll2lfMZ0tv97mlPvvSCRyD0DOM"

        self.baseurl = self.server.projects.baseurl

    def test_get(self) -> None:
        with open(GET_XML, "rb") as f:
            response_xml = f.read().decode("utf-8")
        with requests_mock.mock() as m:
            m.get(self.baseurl, text=response_xml)
            all_projects, pagination_item = self.server.projects.get()

        self.assertEqual(3, pagination_item.total_available)
        self.assertEqual("ee8c6e70-43b6-11e6-af4f-f7b0d8e20760", all_projects[0].id)
        self.assertEqual("default", all_projects[0].name)
        self.assertEqual("The default project that was automatically created by Tableau.", all_projects[0].description)
        self.assertEqual("ManagedByOwner", all_projects[0].content_permissions)
        self.assertEqual(None, all_projects[0].parent_id)
        self.assertEqual("dd2239f6-ddf1-4107-981a-4cf94e415794", all_projects[0].owner_id)

        self.assertEqual("1d0304cd-3796-429f-b815-7258370b9b74", all_projects[1].id)
        self.assertEqual("Tableau", all_projects[1].name)
        self.assertEqual("ManagedByOwner", all_projects[1].content_permissions)
        self.assertEqual(None, all_projects[1].parent_id)
        self.assertEqual("2a47bbf8-8900-4ebb-b0a4-2723bd7c46c3", all_projects[1].owner_id)

        self.assertEqual("4cc52973-5e3a-4d1f-a4fb-5b5f73796edf", all_projects[2].id)
        self.assertEqual("Tableau > Child 1", all_projects[2].name)
        self.assertEqual("ManagedByOwner", all_projects[2].content_permissions)
        self.assertEqual("1d0304cd-3796-429f-b815-7258370b9b74", all_projects[2].parent_id)
        self.assertEqual("dd2239f6-ddf1-4107-981a-4cf94e415794", all_projects[2].owner_id)

    def test_get_before_signin(self) -> None:
        self.server._auth_token = None
        self.assertRaises(TSC.NotSignedInError, self.server.projects.get)

    def test_delete(self) -> None:
        with requests_mock.mock() as m:
            m.delete(self.baseurl + "/ee8c6e70-43b6-11e6-af4f-f7b0d8e20760", status_code=204)
            self.server.projects.delete("ee8c6e70-43b6-11e6-af4f-f7b0d8e20760")

    def test_delete_missing_id(self) -> None:
        self.assertRaises(ValueError, self.server.projects.delete, "")

    def test_update(self) -> None:
        with open(UPDATE_XML, "rb") as f:
            response_xml = f.read().decode("utf-8")
        with requests_mock.mock() as m:
            m.put(self.baseurl + "/1d0304cd-3796-429f-b815-7258370b9b74", text=response_xml)
            single_project = TSC.ProjectItem(
                name="Test Project",
                content_permissions="LockedToProject",
                description="Project created for testing",
                parent_id="9a8f2265-70f3-4494-96c5-e5949d7a1120",
            )
            single_project._id = "1d0304cd-3796-429f-b815-7258370b9b74"
            single_project = self.server.projects.update(single_project)

        self.assertEqual("1d0304cd-3796-429f-b815-7258370b9b74", single_project.id)
        self.assertEqual("Test Project", single_project.name)
        self.assertEqual("Project created for testing", single_project.description)
        self.assertEqual("LockedToProject", single_project.content_permissions)
        self.assertEqual("9a8f2265-70f3-4494-96c5-e5949d7a1120", single_project.parent_id)

    def test_content_permission_locked_to_project_without_nested(self) -> None:
        with open(SET_CONTENT_PERMISSIONS_XML, "rb") as f:
            response_xml = f.read().decode("utf-8")
        with requests_mock.mock() as m:
            m.put(self.baseurl + "/cb3759e5-da4a-4ade-b916-7e2b4ea7ec86", text=response_xml)
            project_item = TSC.ProjectItem(
                name="Test Project Permissions",
                content_permissions="LockedToProjectWithoutNested",
                description="Project created for testing",
                parent_id="7687bc43-a543-42f3-b86f-80caed03a813",
            )
            project_item._id = "cb3759e5-da4a-4ade-b916-7e2b4ea7ec86"
            project_item = self.server.projects.update(project_item)
        self.assertEqual("cb3759e5-da4a-4ade-b916-7e2b4ea7ec86", project_item.id)
        self.assertEqual("Test Project Permissions", project_item.name)
        self.assertEqual("Project created for testing", project_item.description)
        self.assertEqual("LockedToProjectWithoutNested", project_item.content_permissions)
        self.assertEqual("7687bc43-a543-42f3-b86f-80caed03a813", project_item.parent_id)

    def test_update_datasource_default_permission(self) -> None:
        response_xml = read_xml_asset(UPDATE_DATASOURCE_DEFAULT_PERMISSIONS_XML)
        with requests_mock.mock() as m:
            m.put(
                self.baseurl + "/b4065286-80f0-11ea-af1b-cb7191f48e45/default-permissions/datasources",
                text=response_xml,
            )
            project = TSC.ProjectItem("test-project")
            project._id = "b4065286-80f0-11ea-af1b-cb7191f48e45"

            group = TSC.GroupItem("test-group")
            group._id = "b4488bce-80f0-11ea-af1c-976d0c1dab39"

            capabilities = {TSC.Permission.Capability.ExportXml: TSC.Permission.Mode.Deny}

            rules = [TSC.PermissionsRule(grantee=GroupItem.as_reference(group._id), capabilities=capabilities)]

            new_rules = self.server.projects.update_datasource_default_permissions(project, rules)

        self.assertEqual("b4488bce-80f0-11ea-af1c-976d0c1dab39", new_rules[0].grantee.id)

        updated_capabilities = new_rules[0].capabilities
        self.assertEqual(4, len(updated_capabilities))
        self.assertEqual("Deny", updated_capabilities["ExportXml"])
        self.assertEqual("Allow", updated_capabilities["Read"])
        self.assertEqual("Allow", updated_capabilities["Write"])
        self.assertEqual("Allow", updated_capabilities["Connect"])

    def test_update_missing_id(self) -> None:
        single_project = TSC.ProjectItem("test")
        self.assertRaises(TSC.MissingRequiredFieldError, self.server.projects.update, single_project)

    def test_create(self) -> None:
        with open(CREATE_XML, "rb") as f:
            response_xml = f.read().decode("utf-8")
        with requests_mock.mock() as m:
            m.post(self.baseurl, text=response_xml)
            new_project = TSC.ProjectItem(name="Test Project", description="Project created for testing")
            new_project.content_permissions = "ManagedByOwner"
            new_project.parent_id = "9a8f2265-70f3-4494-96c5-e5949d7a1120"
            new_project = self.server.projects.create(new_project)

        self.assertEqual("ccbea03f-77c4-4209-8774-f67bc59c3cef", new_project.id)
        self.assertEqual("Test Project", new_project.name)
        self.assertEqual("Project created for testing", new_project.description)
        self.assertEqual("ManagedByOwner", new_project.content_permissions)
        self.assertEqual("9a8f2265-70f3-4494-96c5-e5949d7a1120", new_project.parent_id)

    def test_create_missing_name(self) -> None:
        self.assertRaises(ValueError, TSC.ProjectItem, "")

    def test_populate_permissions(self) -> None:
        with open(asset(POPULATE_PERMISSIONS_XML), "rb") as f:
            response_xml = f.read().decode("utf-8")
        with requests_mock.mock() as m:
            m.get(self.baseurl + "/0448d2ed-590d-4fa0-b272-a2a8a24555b5/permissions", text=response_xml)
            single_project = TSC.ProjectItem("Project3")
            single_project._id = "0448d2ed-590d-4fa0-b272-a2a8a24555b5"

            self.server.projects.populate_permissions(single_project)
            permissions = single_project.permissions

            self.assertEqual(permissions[0].grantee.tag_name, "group")
            self.assertEqual(permissions[0].grantee.id, "c8f2773a-c83a-11e8-8c8f-33e6d787b506")
            self.assertDictEqual(
                permissions[0].capabilities,
                {
                    TSC.Permission.Capability.Write: TSC.Permission.Mode.Allow,
                    TSC.Permission.Capability.Read: TSC.Permission.Mode.Allow,
                },
            )

    def test_populate_workbooks(self) -> None:
        response_xml = read_xml_asset(POPULATE_WORKBOOK_DEFAULT_PERMISSIONS_XML)
        with requests_mock.mock() as m:
            m.get(
                self.baseurl + "/9dbd2263-16b5-46e1-9c43-a76bb8ab65fb/default-permissions/workbooks", text=response_xml
            )
            single_project = TSC.ProjectItem("test", "1d0304cd-3796-429f-b815-7258370b9b74")
            single_project._owner_id = "dd2239f6-ddf1-4107-981a-4cf94e415794"
            single_project._id = "9dbd2263-16b5-46e1-9c43-a76bb8ab65fb"

            self.server.projects.populate_workbook_default_permissions(single_project)
            permissions = single_project.default_workbook_permissions

        rule1 = permissions.pop()

        self.assertEqual("c8f2773a-c83a-11e8-8c8f-33e6d787b506", rule1.grantee.id)
        self.assertEqual("group", rule1.grantee.tag_name)
        self.assertDictEqual(
            rule1.capabilities,
            {
                TSC.Permission.Capability.Write: TSC.Permission.Mode.Allow,
                TSC.Permission.Capability.Read: TSC.Permission.Mode.Allow,
                TSC.Permission.Capability.Filter: TSC.Permission.Mode.Allow,
                TSC.Permission.Capability.ChangePermissions: TSC.Permission.Mode.Allow,
                TSC.Permission.Capability.WebAuthoring: TSC.Permission.Mode.Allow,
                TSC.Permission.Capability.ExportData: TSC.Permission.Mode.Allow,
                TSC.Permission.Capability.ExportXml: TSC.Permission.Mode.Allow,
                TSC.Permission.Capability.ExportImage: TSC.Permission.Mode.Allow,
                TSC.Permission.Capability.Delete: TSC.Permission.Mode.Deny,
                TSC.Permission.Capability.ShareView: TSC.Permission.Mode.Allow,
                TSC.Permission.Capability.ViewUnderlyingData: TSC.Permission.Mode.Deny,
                TSC.Permission.Capability.ViewComments: TSC.Permission.Mode.Allow,
                TSC.Permission.Capability.AddComment: TSC.Permission.Mode.Allow,
                TSC.Permission.Capability.ChangeHierarchy: TSC.Permission.Mode.Allow,
            },
        )

    def test_delete_permission(self) -> None:
        with open(asset(POPULATE_PERMISSIONS_XML), "rb") as f:
            response_xml = f.read().decode("utf-8")
        with requests_mock.mock() as m:
            m.get(self.baseurl + "/0448d2ed-590d-4fa0-b272-a2a8a24555b5/permissions", text=response_xml)

            single_group = TSC.GroupItem("Group1")
            single_group._id = "c8f2773a-c83a-11e8-8c8f-33e6d787b506"

            single_project = TSC.ProjectItem("Project3")
            single_project._id = "0448d2ed-590d-4fa0-b272-a2a8a24555b5"

            self.server.projects.populate_permissions(single_project)
            permissions = single_project.permissions

            capabilities = {}

            for permission in permissions:
                if permission.grantee.tag_name == "group":
                    if permission.grantee.id == single_group._id:
                        capabilities = permission.capabilities

            rules = TSC.PermissionsRule(grantee=GroupItem.as_reference(single_group._id), capabilities=capabilities)

            endpoint = "{}/permissions/groups/{}".format(single_project._id, single_group._id)
            m.delete("{}/{}/Read/Allow".format(self.baseurl, endpoint), status_code=204)
            m.delete("{}/{}/Write/Allow".format(self.baseurl, endpoint), status_code=204)
            self.server.projects.delete_permission(item=single_project, rules=rules)

    def test_delete_workbook_default_permission(self) -> None:
        with open(asset(POPULATE_WORKBOOK_DEFAULT_PERMISSIONS_XML), "rb") as f:
            response_xml = f.read().decode("utf-8")

        with requests_mock.mock() as m:
            m.get(
                self.baseurl + "/9dbd2263-16b5-46e1-9c43-a76bb8ab65fb/default-permissions/workbooks", text=response_xml
            )

            single_group = TSC.GroupItem("Group1")
            single_group._id = "c8f2773a-c83a-11e8-8c8f-33e6d787b506"

            single_project = TSC.ProjectItem("test", "1d0304cd-3796-429f-b815-7258370b9b74")
            single_project._owner_id = "dd2239f6-ddf1-4107-981a-4cf94e415794"
            single_project._id = "9dbd2263-16b5-46e1-9c43-a76bb8ab65fb"

            self.server.projects.populate_workbook_default_permissions(single_project)
            permissions = single_project.default_workbook_permissions

            capabilities = {
                # View
                TSC.Permission.Capability.Read: TSC.Permission.Mode.Allow,
                TSC.Permission.Capability.ExportImage: TSC.Permission.Mode.Allow,
                TSC.Permission.Capability.ExportData: TSC.Permission.Mode.Allow,
                TSC.Permission.Capability.ViewComments: TSC.Permission.Mode.Allow,
                TSC.Permission.Capability.AddComment: TSC.Permission.Mode.Allow,
                # Interact/Edit
                TSC.Permission.Capability.Filter: TSC.Permission.Mode.Allow,
                TSC.Permission.Capability.ViewUnderlyingData: TSC.Permission.Mode.Deny,
                TSC.Permission.Capability.ShareView: TSC.Permission.Mode.Allow,
                TSC.Permission.Capability.WebAuthoring: TSC.Permission.Mode.Allow,
                # Edit
                TSC.Permission.Capability.Write: TSC.Permission.Mode.Allow,
                TSC.Permission.Capability.ExportXml: TSC.Permission.Mode.Allow,
                TSC.Permission.Capability.ChangeHierarchy: TSC.Permission.Mode.Allow,
                TSC.Permission.Capability.Delete: TSC.Permission.Mode.Deny,
                TSC.Permission.Capability.ChangePermissions: TSC.Permission.Mode.Allow,
            }

            rules = TSC.PermissionsRule(grantee=GroupItem.as_reference(single_group._id), capabilities=capabilities)

            endpoint = "{}/default-permissions/workbooks/groups/{}".format(single_project._id, single_group._id)
            m.delete("{}/{}/Read/Allow".format(self.baseurl, endpoint), status_code=204)
            m.delete("{}/{}/ExportImage/Allow".format(self.baseurl, endpoint), status_code=204)
            m.delete("{}/{}/ExportData/Allow".format(self.baseurl, endpoint), status_code=204)
            m.delete("{}/{}/ViewComments/Allow".format(self.baseurl, endpoint), status_code=204)
            m.delete("{}/{}/AddComment/Allow".format(self.baseurl, endpoint), status_code=204)
            m.delete("{}/{}/Filter/Allow".format(self.baseurl, endpoint), status_code=204)
            m.delete("{}/{}/ViewUnderlyingData/Deny".format(self.baseurl, endpoint), status_code=204)
            m.delete("{}/{}/ShareView/Allow".format(self.baseurl, endpoint), status_code=204)
            m.delete("{}/{}/WebAuthoring/Allow".format(self.baseurl, endpoint), status_code=204)
            m.delete("{}/{}/Write/Allow".format(self.baseurl, endpoint), status_code=204)
            m.delete("{}/{}/ExportXml/Allow".format(self.baseurl, endpoint), status_code=204)
            m.delete("{}/{}/ChangeHierarchy/Allow".format(self.baseurl, endpoint), status_code=204)
            m.delete("{}/{}/Delete/Deny".format(self.baseurl, endpoint), status_code=204)
            m.delete("{}/{}/ChangePermissions/Allow".format(self.baseurl, endpoint), status_code=204)
            self.server.projects.delete_workbook_default_permissions(item=single_project, rule=rules)



####
# This script demonstrates how to use the Tableau Server Client
# to create new projects, both at the root level and how to nest them using
# parent_id.
#
#
# To run the script, you must have installed Python 3.7 or later.
####

import argparse
import logging
import sys

import tableauserverclient as TSC


def create_project(server, project_item, samples=False):
    try:
        project_item = server.projects.create(project_item, samples)
        print("Created a new project called: %s" % project_item.name)
        return project_item
    except TSC.ServerResponseError:
        print("We have already created this project: %s" % project_item.name)
    project_items = server.projects.filter(name=project_item.name)
    return project_items[0]


def main():
    parser = argparse.ArgumentParser(description="Create new projects.")
    # Common options; please keep those in sync across all samples
    parser.add_argument("--server", "-s", required=True, help="server address")
    parser.add_argument("--site", "-S", help="site name")
    parser.add_argument(
        "--token-name", "-p", required=True, help="name of the personal access token used to sign into the server"
    )
    parser.add_argument(
        "--token-value", "-v", required=True, help="value of the personal access token used to sign into the server"
    )
    parser.add_argument(
        "--logging-level",
        "-l",
        choices=["debug", "info", "error"],
        default="error",
        help="desired logging level (set to error by default)",
    )
    # Options specific to this sample
    # This sample has no additional options, yet. If you add some, please add them here

    args = parser.parse_args()

    # Set logging level based on user input, or error by default
    logging_level = getattr(logging, args.logging_level.upper())
    logging.basicConfig(level=logging_level)

    tableau_auth = TSC.PersonalAccessTokenAuth(args.token_name, args.token_value, site_id=args.site)
    server = TSC.Server(args.server, http_options={"verify": False})

    server.use_server_version()
    with server.auth.sign_in(tableau_auth):
        # Use highest Server REST API version available
        server.use_server_version()

        # Without parent_id specified, projects are created at the top level.
        top_level_project = TSC.ProjectItem(name="Top Level Project")
        top_level_project = create_project(server, top_level_project)

        # Specifying parent_id creates a nested projects.
        child_project = TSC.ProjectItem(name="Child Project", parent_id=top_level_project.id)
        child_project = create_project(server, child_project, samples=True)

        # Projects can be nested at any level.
        grand_child_project = TSC.ProjectItem(name="Grand Child Project", parent_id=child_project.id)
        grand_child_project = create_project(server, grand_child_project)

        # Projects can be updated
        changed_project = server.projects.update(grand_child_project, samples=True)

        server.projects.populate_workbook_default_permissions(changed_project),
        server.projects.populate_flow_default_permissions(changed_project),
        server.projects.populate_lens_default_permissions(changed_project),  # uses same as workbook
        server.projects.populate_datasource_default_permissions(changed_project),
        server.projects.populate_permissions(changed_project)
        # Projects have default permissions set for the object types they contain
        print("Permissions from project {}:".format(changed_project.id))
        print(changed_project.permissions)
        print(
            changed_project.default_workbook_permissions,
            changed_project.default_datasource_permissions,
            changed_project.default_lens_permissions,
            changed_project.default_flow_permissions,
        )


if __name__ == "__main__":
    main()



from .column_item import ColumnItem
from .connection_credentials import ConnectionCredentials
from .connection_item import ConnectionItem
from .custom_view_item import CustomViewItem
from .data_acceleration_report_item import DataAccelerationReportItem
from .data_alert_item import DataAlertItem
from .database_item import DatabaseItem
from .datasource_item import DatasourceItem
from .dqw_item import DQWItem
from .exceptions import UnpopulatedPropertyError
from .favorites_item import FavoriteItem
from .fileupload_item import FileuploadItem
from .flow_item import FlowItem
from .flow_run_item import FlowRunItem
from .group_item import GroupItem
from .interval_item import (
    IntervalItem,
    DailyInterval,
    WeeklyInterval,
    MonthlyInterval,
    HourlyInterval,
)
from .job_item import JobItem, BackgroundJobItem
from .metric_item import MetricItem
from .pagination_item import PaginationItem
from .permissions_item import PermissionsRule, Permission
from .project_item import ProjectItem
from .revision_item import RevisionItem
from .schedule_item import ScheduleItem
from .server_info_item import ServerInfoItem
from .site_item import SiteItem
from .subscription_item import SubscriptionItem
from .table_item import TableItem
from .tableau_auth import Credentials, TableauAuth, PersonalAccessTokenAuth
from .tableau_types import Resource, TableauItem, plural_type
from .tag_item import TagItem
from .target import Target
from .task_item import TaskItem
from .user_item import UserItem
from .view_item import ViewItem
from .webhook_item import WebhookItem
from .workbook_item import WorkbookItem



import re
import unittest

import requests_mock

import tableauserverclient as TSC


class SortTests(unittest.TestCase):
    def setUp(self):
        self.server = TSC.Server("http://test", False)
        self.server.version = "3.10"
        self.server._site_id = "dad65087-b08b-4603-af4e-2887b8aafc67"
        self.server._auth_token = "j80k54ll2lfMZ0tv97mlPvvSCRyD0DOM"
        self.baseurl = self.server.workbooks.baseurl

    def test_empty_filter(self):
        self.assertRaises(TypeError, TSC.Filter, "")

    def test_filter_equals(self):
        with requests_mock.mock() as m:
            m.get(requests_mock.ANY)
            url = "http://test/api/2.3/sites/dad65087-b08b-4603-af4e-2887b8aafc67/workbooks"
            opts = TSC.RequestOptions(pagesize=13, pagenumber=13)
            opts.filter.add(TSC.Filter(TSC.RequestOptions.Field.Name, TSC.RequestOptions.Operator.Equals, "Superstore"))

            resp = self.server.workbooks.get_request(url, request_object=opts)

            self.assertTrue(re.search("pagenumber=13", resp.request.query))
            self.assertTrue(re.search("pagesize=13", resp.request.query))
            self.assertTrue(re.search("filter=name%3aeq%3asuperstore", resp.request.query))

    def test_filter_equals_list(self):
        with self.assertRaises(ValueError) as cm:
            TSC.Filter(TSC.RequestOptions.Field.Tags, TSC.RequestOptions.Operator.Equals, ["foo", "bar"])

        self.assertEqual("Filter values can only be a list if the operator is 'in'.", str(cm.exception)),

    def test_filter_in(self):
        with requests_mock.mock() as m:
            m.get(requests_mock.ANY)
            url = "http://test/api/2.3/sites/dad65087-b08b-4603-af4e-2887b8aafc67/workbooks"
            opts = TSC.RequestOptions(pagesize=13, pagenumber=13)

            opts.filter.add(
                TSC.Filter(TSC.RequestOptions.Field.Tags, TSC.RequestOptions.Operator.In, ["stocks", "market"])
            )

            resp = self.server.workbooks.get_request(url, request_object=opts)
            self.assertTrue(re.search("pagenumber=13", resp.request.query))
            self.assertTrue(re.search("pagesize=13", resp.request.query))
            self.assertTrue(re.search("filter=tags%3ain%3a%5bstocks%2cmarket%5d", resp.request.query))

    def test_sort_asc(self):
        with requests_mock.mock() as m:
            m.get(requests_mock.ANY)
            url = "http://test/api/2.3/sites/dad65087-b08b-4603-af4e-2887b8aafc67/workbooks"
            opts = TSC.RequestOptions(pagesize=13, pagenumber=13)
            opts.sort.add(TSC.Sort(TSC.RequestOptions.Field.Name, TSC.RequestOptions.Direction.Asc))

            resp = self.server.workbooks.get_request(url, request_object=opts)

            self.assertTrue(re.search("pagenumber=13", resp.request.query))
            self.assertTrue(re.search("pagesize=13", resp.request.query))
            self.assertTrue(re.search("sort=name%3aasc", resp.request.query))

    def test_filter_combo(self):
        with requests_mock.mock() as m:
            m.get(requests_mock.ANY)
            url = "http://test/api/2.3/sites/dad65087-b08b-4603-af4e-2887b8aafc67/users"
            opts = TSC.RequestOptions(pagesize=13, pagenumber=13)

            opts.filter.add(
                TSC.Filter(
                    TSC.RequestOptions.Field.LastLogin,
                    TSC.RequestOptions.Operator.GreaterThanOrEqual,
                    "2017-01-15T00:00:00:00Z",
                )
            )

            opts.filter.add(
                TSC.Filter(TSC.RequestOptions.Field.SiteRole, TSC.RequestOptions.Operator.Equals, "Publisher")
            )

            resp = self.server.workbooks.get_request(url, request_object=opts)

            expected = (
                "pagenumber=13&pagesize=13&filter=lastlogin%3agte%3a"
                "2017-01-15t00%3a00%3a00%3a00z%2csiterole%3aeq%3apublisher"
            )

            self.assertTrue(re.search("pagenumber=13", resp.request.query))
            self.assertTrue(re.search("pagesize=13", resp.request.query))
            self.assertTrue(
                re.search(
                    "filter=lastlogin%3agte%3a2017-01-15t00%3a00%3a00%3a00z%2csiterole%3aeq%3apublisher",
                    resp.request.query,
                )
            )


if __name__ == "__main__":
    unittest.main()



####
# This script demonstrates how to kill all of the running jobs
#
# To run the script, you must have installed Python 3.7 or later.
####

import argparse
import logging

import tableauserverclient as TSC


def main():
    parser = argparse.ArgumentParser(description="Cancel all of the running background jobs.")
    # Common options; please keep those in sync across all samples
    parser.add_argument("--server", "-s", required=True, help="server address")
    parser.add_argument("--site", "-S", help="site name")
    parser.add_argument(
        "--token-name", "-p", required=True, help="name of the personal access token used to sign into the server"
    )
    parser.add_argument(
        "--token-value", "-v", required=True, help="value of the personal access token used to sign into the server"
    )
    parser.add_argument(
        "--logging-level",
        "-l",
        choices=["debug", "info", "error"],
        default="error",
        help="desired logging level (set to error by default)",
    )
    # Options specific to this sample
    # This sample has no additional options, yet. If you add some, please add them here

    args = parser.parse_args()

    # Set logging level based on user input, or error by default
    logging_level = getattr(logging, args.logging_level.upper())
    logging.basicConfig(level=logging_level)

    tableau_auth = TSC.PersonalAccessTokenAuth(args.token_name, args.token_value, site_id=args.site)
    server = TSC.Server(args.server, use_server_version=True)
    with server.auth.sign_in(tableau_auth):
        req = TSC.RequestOptions()

        req.filter.add(TSC.Filter("progress", TSC.RequestOptions.Operator.LessThanOrEqual, 0))
        for job in TSC.Pager(server.jobs, request_opts=req):
            print(server.jobs.cancel(job.id), job.id, job.status, job.type)


if __name__ == "__main__":
    main()



import os.path
import unittest
from contextlib import contextmanager

TEST_ASSET_DIR = os.path.join(os.path.dirname(__file__), "assets")


def asset(filename):
    return os.path.join(TEST_ASSET_DIR, filename)


def read_xml_asset(filename):
    with open(asset(filename), "rb") as f:
        return f.read().decode("utf-8")


def read_xml_assets(*args):
    return map(read_xml_asset, args)


@contextmanager
def mocked_time():
    mock_time = 0

    def sleep_mock(interval):
        nonlocal mock_time
        mock_time += interval

    def get_time():
        return mock_time

    try:
        patch = unittest.mock.patch
    except AttributeError:
        from unittest.mock import patch
    with patch("time.sleep", sleep_mock), patch("time.time", get_time):
        yield get_time



####
# This script demonstrates how to use the Tableau Server Client
# to move a workbook from one project to another. It will find
# a workbook that matches a given name and update it to be in
# the desired project.
#
# To run the script, you must have installed Python 3.7 or later.
####

import argparse
import logging
import urllib.parse

import tableauserverclient as TSC


def main():
    parser = argparse.ArgumentParser(description="Move one workbook from the default project to another.")
    # Common options; please keep those in sync across all samples
    parser.add_argument("--server", "-s", required=True, help="server address")
    parser.add_argument("--site", "-S", help="site name")
    parser.add_argument(
        "--token-name", "-p", required=True, help="name of the personal access token used to sign into the server"
    )
    parser.add_argument(
        "--token-value", "-v", required=True, help="value of the personal access token used to sign into the server"
    )
    parser.add_argument(
        "--logging-level",
        "-l",
        choices=["debug", "info", "error"],
        default="error",
        help="desired logging level (set to error by default)",
    )
    # Options specific to this sample
    parser.add_argument("--workbook-name", "-w", required=True, help="name of workbook to move")
    parser.add_argument("--destination-project", "-d", required=True, help="name of project to move workbook into")

    args = parser.parse_args()

    # Set logging level based on user input, or error by default
    logging_level = getattr(logging, args.logging_level.upper())
    logging.basicConfig(level=logging_level)

    # Step 1: Sign in to server
    tableau_auth = TSC.PersonalAccessTokenAuth(args.token_name, args.token_value, site_id=args.site)
    server = TSC.Server(args.server, use_server_version=True)
    with server.auth.sign_in(tableau_auth):
        # Step 2: Find destination project
        try:
            dest_project = server.projects.filter(name=urllib.parse.quote_plus(args.destination_project))[0]
        except IndexError:
            raise LookupError(f"No project named {args.destination_project} found.")

        # Step 3: Query workbook to move
        try:
            workbook = server.workbooks.filter(name=urllib.parse.quote_plus(args.workbook_name))[0]
        except IndexError:
            raise LookupError(f"No workbook named {args.workbook_name} found")

        # Step 4: Update workbook with new project id
        workbook.project_id = dest_project.id
        target_workbook = server.workbooks.update(workbook)
        print(f"New project: {target_workbook.project_name}")


if __name__ == "__main__":
    main()



import re

from defusedxml.ElementTree import fromstring

OLD_NAMESPACE = "http://tableausoftware.com/api"
NEW_NAMESPACE = "http://tableau.com/api"
NAMESPACE_RE = re.compile(r"\{(.*?)\}")


class UnknownNamespaceError(Exception):
    pass


class Namespace(object):
    def __init__(self):
        self._namespace = {"t": NEW_NAMESPACE}
        self._detected = False

    def __call__(self):
        return self._namespace

    def detect(self, xml):
        if self._detected:
            return

        if not xml.startswith(b"<?xml"):
            return  # Not an xml file, don't detect anything

        root = fromstring(xml)
        matches = NAMESPACE_RE.match(root.tag)
        if matches:
            detected_ns = matches.group(1)
            if detected_ns in (OLD_NAMESPACE, NEW_NAMESPACE):
                self._namespace = {"t": detected_ns}
                self._detected = True
            else:
                raise UnknownNamespaceError(detected_ns)



import logging
from typing import List, Optional, Tuple

from .endpoint import QuerysetEndpoint, api
from .exceptions import MissingRequiredFieldError
from tableauserverclient.models import CustomViewItem, PaginationItem
from tableauserverclient.server import RequestFactory, RequestOptions, ImageRequestOptions

logger = logging.getLogger("tableau.endpoint.custom_views")

"""
Get a list of custom views on a site
get the details of a custom view
download an image of a custom view.
Delete a custom view 
update the name or owner of a custom view.
"""


class CustomViews(QuerysetEndpoint):
    def __init__(self, parent_srv):
        super(CustomViews, self).__init__(parent_srv)

    @property
    def baseurl(self) -> str:
        return "{0}/sites/{1}/customviews".format(self.parent_srv.baseurl, self.parent_srv.site_id)

    """
    If the request has no filter parameters: Administrators will see all custom views. 
        Other users will see only custom views that they own.
    If the filter parameters include ownerId: Users will see only custom views that they own.
    If the filter parameters include viewId and/or workbookId, and don't include ownerId:
        Users will see those custom views that they have Write and WebAuthoring permissions for.
    If site user visibility is not set to Limited, the Users will see those custom views that are "public",
     meaning the value of their shared attribute is true.
    If site user visibility is set to Limited, ????
    """

    @api(version="3.18")
    def get(self, req_options: Optional["RequestOptions"] = None) -> Tuple[List[CustomViewItem], PaginationItem]:
        logger.info("Querying all custom views on site")
        url = self.baseurl
        server_response = self.get_request(url, req_options)
        pagination_item = PaginationItem.from_response(server_response.content, self.parent_srv.namespace)
        all_view_items = CustomViewItem.list_from_response(server_response.content, self.parent_srv.namespace)
        return all_view_items, pagination_item

    @api(version="3.18")
    def get_by_id(self, view_id: str) -> Optional[CustomViewItem]:
        if not view_id:
            error = "Custom view item missing ID."
            raise MissingRequiredFieldError(error)
        logger.info("Querying custom view (ID: {0})".format(view_id))
        url = "{0}/{1}".format(self.baseurl, view_id)
        server_response = self.get_request(url)
        return CustomViewItem.from_response(server_response.content, self.parent_srv.namespace)

    @api(version="3.18")
    def populate_image(self, view_item: CustomViewItem, req_options: Optional["ImageRequestOptions"] = None) -> None:
        if not view_item.id:
            error = "Custom View item missing ID."
            raise MissingRequiredFieldError(error)

        def image_fetcher():
            return self._get_view_image(view_item, req_options)

        view_item._set_image(image_fetcher)
        logger.info("Populated image for custom view (ID: {0})".format(view_item.id))

    def _get_view_image(self, view_item: CustomViewItem, req_options: Optional["ImageRequestOptions"]) -> bytes:
        url = "{0}/{1}/image".format(self.baseurl, view_item.id)
        server_response = self.get_request(url, req_options)
        image = server_response.content
        return image

    """
    Not yet implemented: pdf or csv exports
    """

    @api(version="3.18")
    def update(self, view_item: CustomViewItem) -> Optional[CustomViewItem]:
        if not view_item.id:
            error = "Custom view item missing ID."
            raise MissingRequiredFieldError(error)
        if not (view_item.owner or view_item.name or view_item.shared):
            logger.debug("No changes to make")
            return view_item

        # Update the custom view owner or name
        url = "{0}/{1}".format(self.baseurl, view_item.id)
        update_req = RequestFactory.CustomView.update_req(view_item)
        server_response = self.put_request(url, update_req)
        logger.info("Updated custom view (ID: {0})".format(view_item.id))
        return CustomViewItem.from_response(server_response.content, self.parent_srv.namespace)

    # Delete 1 view by id
    @api(version="3.19")
    def delete(self, view_id: str) -> None:
        if not view_id:
            error = "Custom View ID undefined."
            raise ValueError(error)
        url = "{0}/{1}".format(self.baseurl, view_id)
        self.delete_request(url)
        logger.info("Deleted single custom view (ID: {0})".format(view_id))



####
# This script demonstrates how to use the Tableau Server Client
# to publish a workbook to a Tableau server. It will publish
# a specified workbook to the 'default' project of the given server.
#
# Note: The REST API publish process cannot automatically include
# extracts or other resources that the workbook uses. Therefore,
# a .twb file with data from a local computer cannot be published,
# unless packaged into a .twbx file.
#
# For more information, refer to the documentations on 'Publish Workbook'
# (https://onlinehelp.tableau.com/current/api/rest_api/en-us/help.htm)
#
# To run the script, you must have installed Python 3.7 or later.
####

import argparse
import logging

import tableauserverclient as TSC
from tableauserverclient import ConnectionCredentials, ConnectionItem


def main():
    parser = argparse.ArgumentParser(description="Publish a workbook to server.")
    # Common options; please keep those in sync across all samples
    parser.add_argument("--server", "-s", required=True, help="server address")
    parser.add_argument("--site", "-S", help="site name")
    parser.add_argument(
        "--token-name", "-p", required=True, help="name of the personal access token used to sign into the server"
    )
    parser.add_argument(
        "--token-value", "-v", required=True, help="value of the personal access token used to sign into the server"
    )
    parser.add_argument(
        "--logging-level",
        "-l",
        choices=["debug", "info", "error"],
        default="error",
        help="desired logging level (set to error by default)",
    )
    # Options specific to this sample
    parser.add_argument("--file", "-f", required=True, help="local filepath of the workbook to publish")
    parser.add_argument("--as-job", "-a", help="Publishing asynchronously", action="store_true")
    parser.add_argument("--skip-connection-check", "-c", help="Skip live connection check", action="store_true")

    args = parser.parse_args()

    # Set logging level based on user input, or error by default
    logging_level = getattr(logging, args.logging_level.upper())
    logging.basicConfig(level=logging_level)

    # Step 1: Sign in to server.
    tableau_auth = TSC.PersonalAccessTokenAuth(args.token_name, args.token_value, site_id=args.site)
    server = TSC.Server(args.server, use_server_version=True)
    with server.auth.sign_in(tableau_auth):
        # Step 2: Get all the projects on server, then look for the default one.
        all_projects, pagination_item = server.projects.get()
        default_project = next((project for project in all_projects if project.is_default()), None)

        connection1 = ConnectionItem()
        connection1.server_address = "mssql.test.com"
        connection1.connection_credentials = ConnectionCredentials("test", "password", True)

        connection2 = ConnectionItem()
        connection2.server_address = "postgres.test.com"
        connection2.server_port = "5432"
        connection2.connection_credentials = ConnectionCredentials("test", "password", True)

        all_connections = list()
        all_connections.append(connection1)
        all_connections.append(connection2)

        # Step 3: If default project is found, form a new workbook item and publish.
        overwrite_true = TSC.Server.PublishMode.Overwrite
        if default_project is not None:
            new_workbook = TSC.WorkbookItem(default_project.id)
            if args.as_job:
                new_job = server.workbooks.publish(
                    new_workbook,
                    args.file,
                    overwrite_true,
                    connections=all_connections,
                    as_job=args.as_job,
                    skip_connection_check=args.skip_connection_check,
                )
                print("Workbook published. JOB ID: {0}".format(new_job.id))
            else:
                new_workbook = server.workbooks.publish(
                    new_workbook,
                    args.file,
                    overwrite_true,
                    connections=all_connections,
                    as_job=args.as_job,
                    skip_connection_check=args.skip_connection_check,
                )
                print("Workbook published. ID: {0}".format(new_workbook.id))
        else:
            error = "The default project could not be found."
            raise LookupError(error)


if __name__ == "__main__":
    main()



####
# This script demonstrates how to use the Tableau Server Client
# to interact with webhooks. It explores the different
# functions that the Server API supports on webhooks.
#
# With no flags set, this sample will query all webhooks,
# pick one webhook and print the name of the webhook.
# Adding flags will demonstrate the specific feature
# on top of the general operations.
####

import argparse
import logging
import os.path

import tableauserverclient as TSC


def main():
    parser = argparse.ArgumentParser(description="Explore webhook functions supported by the Server API.")
    # Common options; please keep those in sync across all samples
    parser.add_argument("--server", "-s", required=True, help="server address")
    parser.add_argument("--site", "-S", help="site name")
    parser.add_argument(
        "--token-name", "-p", required=True, help="name of the personal access token used to sign into the server"
    )
    parser.add_argument(
        "--token-value", "-v", required=True, help="value of the personal access token used to sign into the server"
    )
    parser.add_argument(
        "--logging-level",
        "-l",
        choices=["debug", "info", "error"],
        default="error",
        help="desired logging level (set to error by default)",
    )
    # Options specific to this sample
    parser.add_argument("--create", help="create a webhook")
    parser.add_argument("--delete", help="delete a webhook", action="store_true")

    args = parser.parse_args()

    # Set logging level based on user input, or error by default
    logging_level = getattr(logging, args.logging_level.upper())
    logging.basicConfig(level=logging_level)

    # SIGN IN
    tableau_auth = TSC.PersonalAccessTokenAuth(args.token_name, args.token_value, site_id=args.site)
    server = TSC.Server(args.server, use_server_version=True)
    with server.auth.sign_in(tableau_auth):
        # Create webhook if create flag is set (-create, -c)
        if args.create:
            new_webhook = TSC.WebhookItem()
            new_webhook.name = args.create
            new_webhook.url = "https://ifttt.com/maker-url"
            new_webhook.event = "datasource-created"
            print(new_webhook)
            new_webhook = server.webhooks.create(new_webhook)
            print("Webhook created. ID: {}".format(new_webhook.id))

        # Gets all webhook items
        all_webhooks, pagination_item = server.webhooks.get()
        print("\nThere are {} webhooks on site: ".format(pagination_item.total_available))
        print([webhook.name for webhook in all_webhooks])

        if all_webhooks:
            # Pick one webhook from the list and delete it
            sample_webhook = all_webhooks[0]
            # sample_webhook.delete()
            print("+++" + sample_webhook.name)

            if args.delete:
                print("Deleting webhook " + sample_webhook.name)
                server.webhooks.delete(sample_webhook.id)


if __name__ == "__main__":
    main()



from defusedxml.ElementTree import fromstring

from tableauserverclient.datetime_helpers import parse_datetime


class DQWItem(object):
    class WarningType:
        WARNING = "WARNING"
        DEPRECATED = "DEPRECATED"
        STALE = "STALE"
        SENSITIVE_DATA = "SENSITIVE_DATA"
        MAINTENANCE = "MAINTENANCE"

    def __init__(self, warning_type="WARNING", message=None, active=True, severe=False):
        self._id = None
        # content related
        self._content_id = None
        self._content_type = None

        # DQW related
        self.warning_type = warning_type
        self.message = message
        self.active = active
        self.severe = severe
        self._created_at = None
        self._updated_at = None

        # owner
        self._owner_display_name = None
        self._owner_id = None

    @property
    def id(self):
        return self._id

    @property
    def content_id(self):
        return self._content_id

    @property
    def content_type(self):
        return self._content_type

    @property
    def owner_display_name(self):
        return self._owner_display_name

    @property
    def owner_id(self):
        return self._owner_id

    @property
    def warning_type(self):
        return self._warning_type

    @warning_type.setter
    def warning_type(self, value):
        self._warning_type = value

    @property
    def message(self):
        return self._message

    @message.setter
    def message(self, value):
        self._message = value

    @property
    def active(self):
        return self._active

    @active.setter
    def active(self, value):
        self._active = value

    @property
    def severe(self):
        return self._severe

    @severe.setter
    def severe(self, value):
        self._severe = value

    @property
    def created_at(self):
        return self._created_at

    @created_at.setter
    def created_at(self, value):
        self._created_at = value

    @property
    def updated_at(self):
        return self._updated_at

    @updated_at.setter
    def updated_at(self, value):
        self._updated_at = value

    @classmethod
    def from_response(cls, resp, ns):
        return cls.from_xml_element(fromstring(resp), ns)

    @classmethod
    def from_xml_element(cls, parsed_response, ns):
        all_dqws = []
        dqw_elem_list = parsed_response.findall(".//t:dataQualityWarning", namespaces=ns)
        for dqw_elem in dqw_elem_list:
            dqw = DQWItem()
            dqw._id = dqw_elem.get("id", None)
            dqw._owner_display_name = dqw_elem.get("userDisplayName", None)
            dqw._content_id = dqw_elem.get("contentId", None)
            dqw._content_type = dqw_elem.get("contentType", None)
            dqw.message = dqw_elem.get("message", None)
            dqw.warning_type = dqw_elem.get("type", None)

            is_active = dqw_elem.get("isActive", None)
            if is_active is not None:
                dqw._active = string_to_bool(is_active)

            is_severe = dqw_elem.get("isSevere", None)
            if is_severe is not None:
                dqw._severe = string_to_bool(is_severe)

            dqw._created_at = parse_datetime(dqw_elem.get("createdAt", None))
            dqw._updated_at = parse_datetime(dqw_elem.get("updatedAt", None))

            owner_id = None
            owner_tag = dqw_elem.find(".//t:owner", namespaces=ns)
            if owner_tag is not None:
                owner_id = owner_tag.get("id", None)
            dqw._owner_id = owner_id

            all_dqws.append(dqw)

        return all_dqws


# Used to convert string represented boolean to a boolean type
def string_to_bool(s):
    return s.lower() == "true"



import logging

from .default_permissions_endpoint import _DefaultPermissionsEndpoint
from .dqw_endpoint import _DataQualityWarningEndpoint
from .endpoint import api, Endpoint
from .exceptions import MissingRequiredFieldError
from .permissions_endpoint import _PermissionsEndpoint
from tableauserverclient.server import RequestFactory
from tableauserverclient.models import DatabaseItem, TableItem, PaginationItem, Resource

logger = logging.getLogger("tableau.endpoint.databases")


class Databases(Endpoint):
    def __init__(self, parent_srv):
        super(Databases, self).__init__(parent_srv)

        self._permissions = _PermissionsEndpoint(parent_srv, lambda: self.baseurl)
        self._default_permissions = _DefaultPermissionsEndpoint(parent_srv, lambda: self.baseurl)
        self._data_quality_warnings = _DataQualityWarningEndpoint(parent_srv, Resource.Database)

    @property
    def baseurl(self):
        return "{0}/sites/{1}/databases".format(self.parent_srv.baseurl, self.parent_srv.site_id)

    @api(version="3.5")
    def get(self, req_options=None):
        logger.info("Querying all databases on site")
        url = self.baseurl
        server_response = self.get_request(url, req_options)
        pagination_item = PaginationItem.from_response(server_response.content, self.parent_srv.namespace)
        all_database_items = DatabaseItem.from_response(server_response.content, self.parent_srv.namespace)
        return all_database_items, pagination_item

    # Get 1 database
    @api(version="3.5")
    def get_by_id(self, database_id):
        if not database_id:
            error = "database ID undefined."
            raise ValueError(error)
        logger.info("Querying single database (ID: {0})".format(database_id))
        url = "{0}/{1}".format(self.baseurl, database_id)
        server_response = self.get_request(url)
        return DatabaseItem.from_response(server_response.content, self.parent_srv.namespace)[0]

    @api(version="3.5")
    def delete(self, database_id):
        if not database_id:
            error = "Database ID undefined."
            raise ValueError(error)
        url = "{0}/{1}".format(self.baseurl, database_id)
        self.delete_request(url)
        logger.info("Deleted single database (ID: {0})".format(database_id))

    @api(version="3.5")
    def update(self, database_item):
        if not database_item.id:
            error = "Database item missing ID."
            raise MissingRequiredFieldError(error)

        url = "{0}/{1}".format(self.baseurl, database_item.id)
        update_req = RequestFactory.Database.update_req(database_item)
        server_response = self.put_request(url, update_req)
        logger.info("Updated database item (ID: {0})".format(database_item.id))
        updated_database = DatabaseItem.from_response(server_response.content, self.parent_srv.namespace)[0]
        return updated_database

    # Not Implemented Yet
    @api(version="99")
    def populate_tables(self, database_item):
        if not database_item.id:
            error = "database item missing ID. database must be retrieved from server first."
            raise MissingRequiredFieldError(error)

        def column_fetcher():
            return self._get_tables_for_database(database_item)

        database_item._set_tables(column_fetcher)
        logger.info("Populated tables for database (ID: {0}".format(database_item.id))

    def _get_tables_for_database(self, database_item):
        url = "{0}/{1}/tables".format(self.baseurl, database_item.id)
        server_response = self.get_request(url)
        tables = TableItem.from_response(server_response.content, self.parent_srv.namespace)
        return tables

    @api(version="3.5")
    def populate_permissions(self, item):
        self._permissions.populate(item)

    @api(version="3.5")
    def update_permission(self, item, rules):
        import warnings

        warnings.warn(
            "Server.databases.update_permission is deprecated, "
            "please use Server.databases.update_permissions instead.",
            DeprecationWarning,
        )
        return self._permissions.update(item, rules)

    @api(version="3.5")
    def update_permissions(self, item, rules):
        return self._permissions.update(item, rules)

    @api(version="3.5")
    def delete_permission(self, item, rules):
        self._permissions.delete(item, rules)

    @api(version="3.5")
    def populate_table_default_permissions(self, item):
        self._default_permissions.populate_default_permissions(item, Resource.Table)

    @api(version="3.5")
    def update_table_default_permissions(self, item):
        return self._default_permissions.update_default_permissions(item, Resource.Table)

    @api(version="3.5")
    def delete_table_default_permissions(self, item):
        self._default_permissions.delete_default_permission(item, Resource.Table)

    @api(version="3.5")
    def populate_dqw(self, item):
        self._data_quality_warnings.populate(item)

    @api(version="3.5")
    def update_dqw(self, item, warning):
        return self._data_quality_warnings.update(item, warning)

    @api(version="3.5")
    def add_dqw(self, item, warning):
        return self._data_quality_warnings.add(item, warning)

    @api(version="3.5")
    def delete_dqw(self, item):
        self._data_quality_warnings.clear(item)



import os
import unittest
from datetime import datetime

import requests_mock

import tableauserverclient as TSC
from tableauserverclient.datetime_helpers import utc
from tableauserverclient.server.endpoint.exceptions import JobFailedException
from ._utils import read_xml_asset, mocked_time

GET_XML = "job_get.xml"
GET_BY_ID_XML = "job_get_by_id.xml"
GET_BY_ID_FAILED_XML = "job_get_by_id_failed.xml"
GET_BY_ID_CANCELLED_XML = "job_get_by_id_cancelled.xml"
GET_BY_ID_INPROGRESS_XML = "job_get_by_id_inprogress.xml"
GET_BY_ID_WORKBOOK = "job_get_by_id_failed_workbook.xml"


class JobTests(unittest.TestCase):
    def setUp(self) -> None:
        self.server = TSC.Server("http://test", False)
        self.server.version = "3.1"

        # Fake signin
        self.server._site_id = "dad65087-b08b-4603-af4e-2887b8aafc67"
        self.server._auth_token = "j80k54ll2lfMZ0tv97mlPvvSCRyD0DOM"

        self.baseurl = self.server.jobs.baseurl

    def test_get(self) -> None:
        response_xml = read_xml_asset(GET_XML)
        with requests_mock.mock() as m:
            m.get(self.baseurl, text=response_xml)
            all_jobs, pagination_item = self.server.jobs.get()
            job = all_jobs[0]
            created_at = datetime(2018, 5, 22, 13, 0, 29, tzinfo=utc)
            started_at = datetime(2018, 5, 22, 13, 0, 37, tzinfo=utc)
            ended_at = datetime(2018, 5, 22, 13, 0, 45, tzinfo=utc)

            self.assertEqual(1, pagination_item.total_available)
            self.assertEqual("2eef4225-aa0c-41c4-8662-a76d89ed7336", job.id)
            self.assertEqual("Success", job.status)
            self.assertEqual("50", job.priority)
            self.assertEqual("single_subscription_notify", job.type)
            self.assertEqual(created_at, job.created_at)
            self.assertEqual(started_at, job.started_at)
            self.assertEqual(ended_at, job.ended_at)

    def test_get_by_id(self) -> None:
        response_xml = read_xml_asset(GET_BY_ID_XML)
        job_id = "2eef4225-aa0c-41c4-8662-a76d89ed7336"
        with requests_mock.mock() as m:
            m.get("{0}/{1}".format(self.baseurl, job_id), text=response_xml)
            job = self.server.jobs.get_by_id(job_id)
            updated_at = datetime(2020, 5, 13, 20, 25, 18, tzinfo=utc)

            self.assertEqual(job_id, job.id)
            self.assertEqual(updated_at, job.updated_at)
            self.assertListEqual(job.notes, ["Job detail notes"])

    def test_get_before_signin(self) -> None:
        self.server._auth_token = None
        self.assertRaises(TSC.NotSignedInError, self.server.jobs.get)

    def test_cancel_id(self) -> None:
        with requests_mock.mock() as m:
            m.put(self.baseurl + "/ee8c6e70-43b6-11e6-af4f-f7b0d8e20760", status_code=204)
            self.server.jobs.cancel("ee8c6e70-43b6-11e6-af4f-f7b0d8e20760")

    def test_cancel_item(self) -> None:
        created_at = datetime(2018, 5, 22, 13, 0, 29, tzinfo=utc)
        started_at = datetime(2018, 5, 22, 13, 0, 37, tzinfo=utc)
        job = TSC.JobItem("ee8c6e70-43b6-11e6-af4f-f7b0d8e20760", "backgroundJob", "0", created_at, started_at, None, 0)
        with requests_mock.mock() as m:
            m.put(self.baseurl + "/ee8c6e70-43b6-11e6-af4f-f7b0d8e20760", status_code=204)
            self.server.jobs.cancel(job)

    def test_wait_for_job_finished(self) -> None:
        # Waiting for an already finished job, directly returns that job's info
        response_xml = read_xml_asset(GET_BY_ID_XML)
        job_id = "2eef4225-aa0c-41c4-8662-a76d89ed7336"
        with mocked_time(), requests_mock.mock() as m:
            m.get("{0}/{1}".format(self.baseurl, job_id), text=response_xml)
            job = self.server.jobs.wait_for_job(job_id)

            self.assertEqual(job_id, job.id)
            self.assertListEqual(job.notes, ["Job detail notes"])

    def test_wait_for_job_failed(self) -> None:
        # Waiting for a failed job raises an exception
        response_xml = read_xml_asset(GET_BY_ID_FAILED_XML)
        job_id = "77d5e57a-2517-479f-9a3c-a32025f2b64d"
        with mocked_time(), requests_mock.mock() as m:
            m.get("{0}/{1}".format(self.baseurl, job_id), text=response_xml)
            with self.assertRaises(JobFailedException):
                self.server.jobs.wait_for_job(job_id)

    def test_wait_for_job_timeout(self) -> None:
        # Waiting for a job which doesn't terminate will throw an exception
        response_xml = read_xml_asset(GET_BY_ID_INPROGRESS_XML)
        job_id = "77d5e57a-2517-479f-9a3c-a32025f2b64d"
        with mocked_time(), requests_mock.mock() as m:
            m.get("{0}/{1}".format(self.baseurl, job_id), text=response_xml)
            with self.assertRaises(TimeoutError):
                self.server.jobs.wait_for_job(job_id, timeout=30)

    def test_get_job_datasource_id(self) -> None:
        response_xml = read_xml_asset(GET_BY_ID_FAILED_XML)
        job_id = "777bf7c4-421d-4b2c-a518-11b90187c545"
        with requests_mock.mock() as m:
            m.get(f"{self.baseurl}/{job_id}", text=response_xml)
            job = self.server.jobs.get_by_id(job_id)
        self.assertEqual(job.datasource_id, "03b9fbec-81f6-4160-ae49-5f9f6d412758")

    def test_get_job_workbook_id(self) -> None:
        response_xml = read_xml_asset(GET_BY_ID_WORKBOOK)
        job_id = "bb1aab79-db54-4e96-9dd3-461d8f081d08"
        with requests_mock.mock() as m:
            m.get(f"{self.baseurl}/{job_id}", text=response_xml)
            job = self.server.jobs.get_by_id(job_id)
        self.assertEqual(job.workbook_id, "5998aaaf-1abe-4d38-b4d9-bc53e85bdd13")



import logging

from .endpoint import api, Endpoint
from .exceptions import MissingRequiredFieldError
from tableauserverclient.server import RequestFactory
from tableauserverclient.models import DataAlertItem, PaginationItem, UserItem

logger = logging.getLogger("tableau.endpoint.dataAlerts")

from typing import List, Optional, TYPE_CHECKING, Tuple, Union


if TYPE_CHECKING:
    from ..server import Server
    from ..request_options import RequestOptions


class DataAlerts(Endpoint):
    def __init__(self, parent_srv: "Server") -> None:
        super(DataAlerts, self).__init__(parent_srv)

    @property
    def baseurl(self) -> str:
        return "{0}/sites/{1}/dataAlerts".format(self.parent_srv.baseurl, self.parent_srv.site_id)

    @api(version="3.2")
    def get(self, req_options: Optional["RequestOptions"] = None) -> Tuple[List[DataAlertItem], PaginationItem]:
        logger.info("Querying all dataAlerts on site")
        url = self.baseurl
        server_response = self.get_request(url, req_options)
        pagination_item = PaginationItem.from_response(server_response.content, self.parent_srv.namespace)
        all_dataAlert_items = DataAlertItem.from_response(server_response.content, self.parent_srv.namespace)
        return all_dataAlert_items, pagination_item

    # Get 1 dataAlert
    @api(version="3.2")
    def get_by_id(self, dataAlert_id: str) -> DataAlertItem:
        if not dataAlert_id:
            error = "dataAlert ID undefined."
            raise ValueError(error)
        logger.info("Querying single dataAlert (ID: {0})".format(dataAlert_id))
        url = "{0}/{1}".format(self.baseurl, dataAlert_id)
        server_response = self.get_request(url)
        return DataAlertItem.from_response(server_response.content, self.parent_srv.namespace)[0]

    @api(version="3.2")
    def delete(self, dataAlert: Union[DataAlertItem, str]) -> None:
        if isinstance(dataAlert, DataAlertItem):
            dataAlert_id = dataAlert.id
        elif isinstance(dataAlert, str):
            dataAlert_id = dataAlert
        else:
            raise TypeError("dataAlert should be a DataAlertItem or a string of an id.")
        if not dataAlert_id:
            error = "Dataalert ID undefined."
            raise ValueError(error)
        # DELETE /api/api-version/sites/site-id/dataAlerts/data-alert-id/users/user-id
        url = "{0}/{1}".format(self.baseurl, dataAlert_id)
        self.delete_request(url)
        logger.info("Deleted single dataAlert (ID: {0})".format(dataAlert_id))

    @api(version="3.2")
    def delete_user_from_alert(self, dataAlert: Union[DataAlertItem, str], user: Union[UserItem, str]) -> None:
        if isinstance(dataAlert, DataAlertItem):
            dataAlert_id = dataAlert.id
        elif isinstance(dataAlert, str):
            dataAlert_id = dataAlert
        else:
            raise TypeError("dataAlert should be a DataAlertItem or a string of an id.")
        if isinstance(user, UserItem):
            user_id = user.id
        elif isinstance(user, str):
            user_id = user
        else:
            raise TypeError("user should be a UserItem or a string of an id.")
        if not dataAlert_id:
            error = "Dataalert ID undefined."
            raise ValueError(error)
        if not user_id:
            error = "User ID undefined."
            raise ValueError(error)
        # DELETE /api/api-version/sites/site-id/dataAlerts/data-alert-id/users/user-id
        url = "{0}/{1}/users/{2}".format(self.baseurl, dataAlert_id, user_id)
        self.delete_request(url)
        logger.info("Deleted User (ID {0}) from dataAlert (ID: {1})".format(user_id, dataAlert_id))

    @api(version="3.2")
    def add_user_to_alert(self, dataAlert_item: DataAlertItem, user: Union[UserItem, str]) -> UserItem:
        if isinstance(user, UserItem):
            user_id = user.id
        elif isinstance(user, str):
            user_id = user
        else:
            raise TypeError("user should be a UserItem or a string of an id.")
        if not dataAlert_item.id:
            error = "Dataalert item missing ID."
            raise MissingRequiredFieldError(error)
        if not user_id:
            error = "User ID undefined."
            raise ValueError(error)
        url = "{0}/{1}/users".format(self.baseurl, dataAlert_item.id)
        update_req = RequestFactory.DataAlert.add_user_to_alert(dataAlert_item, user_id)
        server_response = self.post_request(url, update_req)
        logger.info("Added user (ID {0}) to dataAlert item (ID: {1})".format(user_id, dataAlert_item.id))
        added_user = UserItem.from_response(server_response.content, self.parent_srv.namespace)[0]
        return added_user

    @api(version="3.2")
    def update(self, dataAlert_item: DataAlertItem) -> DataAlertItem:
        if not dataAlert_item.id:
            error = "Dataalert item missing ID."
            raise MissingRequiredFieldError(error)

        url = "{0}/{1}".format(self.baseurl, dataAlert_item.id)
        update_req = RequestFactory.DataAlert.update_req(dataAlert_item)
        server_response = self.put_request(url, update_req)
        logger.info("Updated dataAlert item (ID: {0})".format(dataAlert_item.id))
        updated_dataAlert = DataAlertItem.from_response(server_response.content, self.parent_srv.namespace)[0]
        return updated_dataAlert



import os.path
import unittest

import requests_mock

import tableauserverclient as TSC

TEST_ASSET_DIR = os.path.join(os.path.dirname(__file__), "assets")

GET_XML = os.path.join(TEST_ASSET_DIR, "site_get.xml")
GET_BY_ID_XML = os.path.join(TEST_ASSET_DIR, "site_get_by_id.xml")
GET_BY_NAME_XML = os.path.join(TEST_ASSET_DIR, "site_get_by_name.xml")
UPDATE_XML = os.path.join(TEST_ASSET_DIR, "site_update.xml")
CREATE_XML = os.path.join(TEST_ASSET_DIR, "site_create.xml")


class SiteTests(unittest.TestCase):
    def setUp(self) -> None:
        self.server = TSC.Server("http://test", False)
        self.server.version = "3.10"

        # Fake signin
        self.server._auth_token = "j80k54ll2lfMZ0tv97mlPvvSCRyD0DOM"
        self.server._site_id = "0626857c-1def-4503-a7d8-7907c3ff9d9f"
        self.baseurl = self.server.sites.baseurl

        # sites APIs can only be called on the site being logged in to
        self.logged_in_site = self.server.site_id

    def test_get(self) -> None:
        with open(GET_XML, "rb") as f:
            response_xml = f.read().decode("utf-8")
        with requests_mock.mock() as m:
            m.get(self.baseurl, text=response_xml)
            all_sites, pagination_item = self.server.sites.get()

        self.assertEqual(2, pagination_item.total_available)
        self.assertEqual("dad65087-b08b-4603-af4e-2887b8aafc67", all_sites[0].id)
        self.assertEqual("Active", all_sites[0].state)
        self.assertEqual("Default", all_sites[0].name)
        self.assertEqual("ContentOnly", all_sites[0].admin_mode)
        self.assertEqual(False, all_sites[0].revision_history_enabled)
        self.assertEqual(True, all_sites[0].subscribe_others_enabled)
        self.assertEqual(25, all_sites[0].revision_limit)
        self.assertEqual(None, all_sites[0].num_users)
        self.assertEqual(None, all_sites[0].storage)
        self.assertEqual(True, all_sites[0].cataloging_enabled)
        self.assertEqual(False, all_sites[0].editing_flows_enabled)
        self.assertEqual(False, all_sites[0].scheduling_flows_enabled)
        self.assertEqual(True, all_sites[0].allow_subscription_attachments)
        self.assertEqual("6b7179ba-b82b-4f0f-91ed-812074ac5da6", all_sites[1].id)
        self.assertEqual("Active", all_sites[1].state)
        self.assertEqual("Samples", all_sites[1].name)
        self.assertEqual("ContentOnly", all_sites[1].admin_mode)
        self.assertEqual(False, all_sites[1].revision_history_enabled)
        self.assertEqual(True, all_sites[1].subscribe_others_enabled)
        self.assertEqual(False, all_sites[1].guest_access_enabled)
        self.assertEqual(True, all_sites[1].cache_warmup_enabled)
        self.assertEqual(True, all_sites[1].commenting_enabled)
        self.assertEqual(True, all_sites[1].cache_warmup_enabled)
        self.assertEqual(False, all_sites[1].request_access_enabled)
        self.assertEqual(True, all_sites[1].run_now_enabled)
        self.assertEqual(1, all_sites[1].tier_explorer_capacity)
        self.assertEqual(2, all_sites[1].tier_creator_capacity)
        self.assertEqual(1, all_sites[1].tier_viewer_capacity)
        self.assertEqual(False, all_sites[1].flows_enabled)
        self.assertEqual(None, all_sites[1].data_acceleration_mode)

    def test_get_before_signin(self) -> None:
        self.server._auth_token = None
        self.assertRaises(TSC.NotSignedInError, self.server.sites.get)

    def test_get_by_id(self) -> None:
        with open(GET_BY_ID_XML, "rb") as f:
            response_xml = f.read().decode("utf-8")
        with requests_mock.mock() as m:
            m.get(self.baseurl + "/" + self.logged_in_site, text=response_xml)
            single_site = self.server.sites.get_by_id(self.logged_in_site)

        self.assertEqual(self.logged_in_site, single_site.id)
        self.assertEqual("Active", single_site.state)
        self.assertEqual("Default", single_site.name)
        self.assertEqual("ContentOnly", single_site.admin_mode)
        self.assertEqual(False, single_site.revision_history_enabled)
        self.assertEqual(True, single_site.subscribe_others_enabled)
        self.assertEqual(False, single_site.disable_subscriptions)
        self.assertEqual(False, single_site.data_alerts_enabled)
        self.assertEqual(False, single_site.commenting_mentions_enabled)
        self.assertEqual(True, single_site.catalog_obfuscation_enabled)

    def test_get_by_id_missing_id(self) -> None:
        self.assertRaises(ValueError, self.server.sites.get_by_id, "")

    def test_get_by_name(self) -> None:
        with open(GET_BY_NAME_XML, "rb") as f:
            response_xml = f.read().decode("utf-8")
        with requests_mock.mock() as m:
            m.get(self.baseurl + "/testsite?key=name", text=response_xml)
            single_site = self.server.sites.get_by_name("testsite")

        self.assertEqual(self.logged_in_site, single_site.id)
        self.assertEqual("Active", single_site.state)
        self.assertEqual("testsite", single_site.name)
        self.assertEqual("ContentOnly", single_site.admin_mode)
        self.assertEqual(False, single_site.revision_history_enabled)
        self.assertEqual(True, single_site.subscribe_others_enabled)
        self.assertEqual(False, single_site.disable_subscriptions)

    def test_get_by_name_missing_name(self) -> None:
        self.assertRaises(ValueError, self.server.sites.get_by_name, "")

    def test_update(self) -> None:
        with open(UPDATE_XML, "rb") as f:
            response_xml = f.read().decode("utf-8")
        with requests_mock.mock() as m:
            m.put(self.baseurl + "/" + self.logged_in_site, text=response_xml)
            single_site = TSC.SiteItem(
                name="Tableau",
                content_url="tableau",
                admin_mode=TSC.SiteItem.AdminMode.ContentAndUsers,
                user_quota=15,
                storage_quota=1000,
                disable_subscriptions=True,
                revision_history_enabled=False,
                data_acceleration_mode="disable",
                flow_auto_save_enabled=True,
                web_extraction_enabled=False,
                metrics_content_type_enabled=True,
                notify_site_admins_on_throttle=False,
                authoring_enabled=True,
                custom_subscription_email_enabled=True,
                custom_subscription_email="test@test.com",
                custom_subscription_footer_enabled=True,
                custom_subscription_footer="example_footer",
                ask_data_mode="EnabledByDefault",
                named_sharing_enabled=False,
                mobile_biometrics_enabled=True,
                sheet_image_enabled=False,
                derived_permissions_enabled=True,
                user_visibility_mode="FULL",
                use_default_time_zone=False,
                time_zone="America/Los_Angeles",
                auto_suspend_refresh_enabled=True,
                auto_suspend_refresh_inactivity_window=55,
                tier_creator_capacity=5,
                tier_explorer_capacity=5,
                tier_viewer_capacity=5,
            )
            single_site._id = self.logged_in_site
            self.server.sites.parent_srv = self.server
            single_site = self.server.sites.update(single_site)

        self.assertEqual(self.logged_in_site, single_site.id)
        self.assertEqual("tableau", single_site.content_url)
        self.assertEqual("Suspended", single_site.state)
        self.assertEqual("Tableau", single_site.name)
        self.assertEqual("ContentAndUsers", single_site.admin_mode)
        self.assertEqual(True, single_site.revision_history_enabled)
        self.assertEqual(13, single_site.revision_limit)
        self.assertEqual(True, single_site.disable_subscriptions)
        self.assertEqual(None, single_site.user_quota)
        self.assertEqual(5, single_site.tier_creator_capacity)
        self.assertEqual(5, single_site.tier_explorer_capacity)
        self.assertEqual(5, single_site.tier_viewer_capacity)
        self.assertEqual("disable", single_site.data_acceleration_mode)
        self.assertEqual(True, single_site.flows_enabled)
        self.assertEqual(True, single_site.cataloging_enabled)
        self.assertEqual(True, single_site.flow_auto_save_enabled)
        self.assertEqual(False, single_site.web_extraction_enabled)
        self.assertEqual(True, single_site.metrics_content_type_enabled)
        self.assertEqual(False, single_site.notify_site_admins_on_throttle)
        self.assertEqual(True, single_site.authoring_enabled)
        self.assertEqual(True, single_site.custom_subscription_email_enabled)
        self.assertEqual("test@test.com", single_site.custom_subscription_email)
        self.assertEqual(True, single_site.custom_subscription_footer_enabled)
        self.assertEqual("example_footer", single_site.custom_subscription_footer)
        self.assertEqual("EnabledByDefault", single_site.ask_data_mode)
        self.assertEqual(False, single_site.named_sharing_enabled)
        self.assertEqual(True, single_site.mobile_biometrics_enabled)
        self.assertEqual(False, single_site.sheet_image_enabled)
        self.assertEqual(True, single_site.derived_permissions_enabled)
        self.assertEqual("FULL", single_site.user_visibility_mode)
        self.assertEqual(False, single_site.use_default_time_zone)
        self.assertEqual("America/Los_Angeles", single_site.time_zone)
        self.assertEqual(True, single_site.auto_suspend_refresh_enabled)
        self.assertEqual(55, single_site.auto_suspend_refresh_inactivity_window)

    def test_update_missing_id(self) -> None:
        single_site = TSC.SiteItem("test", "test")
        self.assertRaises(TSC.MissingRequiredFieldError, self.server.sites.update, single_site)

    def test_null_site_quota(self) -> None:
        test_site = TSC.SiteItem("testname", "testcontenturl", tier_explorer_capacity=1, user_quota=None)
        assert test_site.tier_explorer_capacity == 1
        with self.assertRaises(ValueError):
            test_site.user_quota = 1
        test_site.tier_explorer_capacity = None
        test_site.user_quota = 1

    def test_replace_license_tiers_with_user_quota(self) -> None:
        test_site = TSC.SiteItem("testname", "testcontenturl", tier_explorer_capacity=1, user_quota=None)
        assert test_site.tier_explorer_capacity == 1
        with self.assertRaises(ValueError):
            test_site.user_quota = 1
        test_site.replace_license_tiers_with_user_quota(1)
        self.assertEqual(1, test_site.user_quota)
        self.assertIsNone(test_site.tier_explorer_capacity)

    def test_create(self) -> None:
        with open(CREATE_XML, "rb") as f:
            response_xml = f.read().decode("utf-8")
        with requests_mock.mock() as m:
            m.post(self.baseurl, text=response_xml)
            new_site = TSC.SiteItem(
                name="Tableau",
                content_url="tableau",
                admin_mode=TSC.SiteItem.AdminMode.ContentAndUsers,
                user_quota=15,
                storage_quota=1000,
                disable_subscriptions=True,
            )
            new_site = self.server.sites.create(new_site)

        new_site._tier_viewer_capacity = None
        new_site._tier_creator_capacity = None
        new_site._tier_explorer_capacity = None
        self.assertEqual("0626857c-1def-4503-a7d8-7907c3ff9d9f", new_site.id)
        self.assertEqual("tableau", new_site.content_url)
        self.assertEqual("Tableau", new_site.name)
        self.assertEqual("Active", new_site.state)
        self.assertEqual("ContentAndUsers", new_site.admin_mode)
        self.assertEqual(False, new_site.revision_history_enabled)
        self.assertEqual(True, new_site.subscribe_others_enabled)
        self.assertEqual(True, new_site.disable_subscriptions)
        self.assertEqual(15, new_site.user_quota)

    def test_delete(self) -> None:
        with requests_mock.mock() as m:
            m.delete(self.baseurl + "/0626857c-1def-4503-a7d8-7907c3ff9d9f", status_code=204)
            self.server.sites.delete("0626857c-1def-4503-a7d8-7907c3ff9d9f")

    def test_delete_missing_id(self) -> None:
        self.assertRaises(ValueError, self.server.sites.delete, "")

    def test_encrypt(self) -> None:
        with requests_mock.mock() as m:
            m.post(self.baseurl + "/0626857c-1def-4503-a7d8-7907c3ff9d9f/encrypt-extracts", status_code=200)
            self.server.sites.encrypt_extracts("0626857c-1def-4503-a7d8-7907c3ff9d9f")

    def test_recrypt(self) -> None:
        with requests_mock.mock() as m:
            m.post(self.baseurl + "/0626857c-1def-4503-a7d8-7907c3ff9d9f/reencrypt-extracts", status_code=200)
            self.server.sites.re_encrypt_extracts("0626857c-1def-4503-a7d8-7907c3ff9d9f")

    def test_decrypt(self) -> None:
        with requests_mock.mock() as m:
            m.post(self.baseurl + "/0626857c-1def-4503-a7d8-7907c3ff9d9f/decrypt-extracts", status_code=200)
            self.server.sites.decrypt_extracts("0626857c-1def-4503-a7d8-7907c3ff9d9f")



import os

ALLOWED_SPECIAL = (" ", ".", "_", "-")


def to_filename(string_to_sanitize):
    sanitized = (c for c in string_to_sanitize if c.isalnum() or c in ALLOWED_SPECIAL)
    return "".join(sanitized)


def make_download_path(filepath, filename):
    download_path = None

    if filepath is None:
        download_path = filename

    elif os.path.isdir(filepath):
        download_path = os.path.join(filepath, filename)

    else:
        download_path = filepath + os.path.splitext(filename)[1]

    return download_path


def get_file_object_size(file):
    # Returns the size of a file object
    file.seek(0, os.SEEK_END)
    file_size = file.tell()
    file.seek(0)
    return file_size


def get_file_type(file):
    # Tableau workbooks (twb) and data sources (tds) are both stored as xml files.
    # Packaged workbooks (twbx) and data sources (tdsx) are zip files
    # containing original files accompanied with supporting local files.

    # This reference lists magic file signatures: https://www.garykessler.net/library/file_sigs.html
    MAGIC_BYTES = {
        "zip": bytes.fromhex("504b0304"),
        "tde": bytes.fromhex("20020162"),
        "xml": bytes.fromhex("3c3f786d6c20"),
        "hyper": bytes.fromhex("487970657208000001000000"),
    }

    # Peek first bytes of a file
    first_bytes = file.read(32)

    file_type = None
    for ft, signature in MAGIC_BYTES.items():
        if first_bytes.startswith(signature):
            file_type = ft
            break

    # Return pointer back to start
    file.seek(0)

    if file_type is None:
        error = "Unknown file type!"
        raise ValueError(error)

    return file_type



####
# This script demonstrates how to update the data within a published
# live-to-Hyper datasource on server.
#
# The sample is hardcoded against the `World Indicators` dataset and
# expects to receive the LUID of a published datasource containing
# that data. To create such a published datasource, you can use:
#   ./publish_datasource.py --file ../test/assets/World\ Indicators.hyper
# which will print you the LUID of the datasource.
#
# Before running this script, the datasource will contain a region `Europe`.
# After running this script, that region will be gone.
#
####

import argparse
import uuid
import logging

import tableauserverclient as TSC


def main():
    parser = argparse.ArgumentParser(
        description="Delete the `Europe` region from a published `World Indicators` datasource."
    )
    # Common options; please keep those in sync across all samples
    parser.add_argument("--server", "-s", required=True, help="server address")
    parser.add_argument("--site", "-S", help="site name")
    parser.add_argument(
        "--token-name", "-p", required=True, help="name of the personal access token used to sign into the server"
    )
    parser.add_argument(
        "--token-value", "-v", required=True, help="value of the personal access token used to sign into the server"
    )
    parser.add_argument(
        "--logging-level",
        "-l",
        choices=["debug", "info", "error"],
        default="error",
        help="desired logging level (set to error by default)",
    )
    # Options specific to this sample
    parser.add_argument("datasource_id", help="The LUID of the `World Indicators` datasource")

    args = parser.parse_args()

    # Set logging level based on user input, or error by default
    logging_level = getattr(logging, args.logging_level.upper())
    logging.basicConfig(level=logging_level)

    tableau_auth = TSC.PersonalAccessTokenAuth(args.token_name, args.token_value, site_id=args.site)
    server = TSC.Server(args.server, use_server_version=True)
    with server.auth.sign_in(tableau_auth):
        # We use a unique `request_id` for every request.
        # In case the submission of the update job fails, we won't know wether the job was submitted
        # or not. It could be that the server received the request, changed the data, but then the
        # network connection broke down.
        # If you want to have a way to retry, e.g., inserts while making sure they aren't duplicated,
        # you need to use `request_id` for that purpose.
        # In our case, we don't care about retries. And the delete is idempotent anyway.
        # Hence, we simply use a randomly generated request id.
        request_id = str(uuid.uuid4())

        # This action will delete all rows with `Region=Europe` from the published data source.
        # Other actions (inserts, updates, ...) are also available. For more information see
        # https://help.tableau.com/current/api/rest_api/en-us/REST/rest_api_how_to_update_data_to_hyper.htm
        actions = [
            {
                "action": "delete",
                "target-table": "Extract",
                "target-schema": "Extract",
                "condition": {"op": "eq", "target-col": "Region", "const": {"type": "string", "v": "Europe"}},
            }
        ]

        job = server.datasources.update_hyper_data(args.datasource_id, request_id=request_id, actions=actions)

        print(f"Update job posted (ID: {job.id})")
        print("Waiting for job...")
        # `wait_for_job` will throw if the job isn't executed successfully
        job = server.jobs.wait_for_job(job)
        print("Job finished succesfully")


if __name__ == "__main__":
    main()



####
# This script demonstrates how to add default permissions using TSC
# To run the script, you must have installed Python 3.7 or later.
#
# In order to demonstrate adding a new default permission, this sample will create
# a new project and add a new capability to the new project, for the default "All users" group.
#
# Example usage: 'python add_default_permission.py -s
#       https://10ax.online.tableau.com --site devSite123 -u tabby@tableau.com'
####

import argparse
import logging

import tableauserverclient as TSC


def main():
    parser = argparse.ArgumentParser(description="Add workbook default permissions for a given project.")
    # Common options; please keep those in sync across all samples
    parser.add_argument("--server", "-s", required=True, help="server address")
    parser.add_argument("--site", "-S", help="site name")
    parser.add_argument(
        "--token-name", "-p", required=True, help="name of the personal access token used to sign into the server"
    )
    parser.add_argument(
        "--token-value", "-v", required=True, help="value of the personal access token used to sign into the server"
    )
    parser.add_argument(
        "--logging-level",
        "-l",
        choices=["debug", "info", "error"],
        default="error",
        help="desired logging level (set to error by default)",
    )
    # Options specific to this sample
    # This sample has no additional options, yet. If you add some, please add them here

    args = parser.parse_args()

    # Set logging level based on user input, or error by default
    logging_level = getattr(logging, args.logging_level.upper())
    logging.basicConfig(level=logging_level)

    # Sign in to server
    tableau_auth = TSC.PersonalAccessTokenAuth(args.token_name, args.token_value, site_id=args.site)
    server = TSC.Server(args.server, use_server_version=True)
    with server.auth.sign_in(tableau_auth):
        # Create a sample project
        project = TSC.ProjectItem("sample_project")
        project = server.projects.create(project)

        # Query for existing workbook default-permissions
        server.projects.populate_workbook_default_permissions(project)
        default_permissions = project.default_workbook_permissions[0]  # new projects have 1 grantee group

        # Add "ExportXml (Allow)" workbook capability to "All Users" default group if it does not already exist
        if TSC.Permission.Capability.ExportXml not in default_permissions.capabilities:
            new_capabilities = {TSC.Permission.Capability.ExportXml: TSC.Permission.Mode.Allow}

            # Each PermissionRule in the list contains a grantee and a dict of capabilities
            new_rules = [TSC.PermissionsRule(grantee=default_permissions.grantee, capabilities=new_capabilities)]

            new_default_permissions = server.projects.update_workbook_default_permissions(project, new_rules)

            # Print result from adding a new default permission
            for permission in new_default_permissions:
                grantee = permission.grantee
                capabilities = permission.capabilities
                print("\nCapabilities for {0} {1}:".format(grantee.tag_name, grantee.id))

                for capability in capabilities:
                    print("\t{0} - {1}".format(capability, capabilities[capability]))

        # Uncomment lines below to DELETE the new capability and the new project
        # rules_to_delete = TSC.PermissionsRule(
        #     grantee=default_permissions.grantee,
        #     capabilities=new_capabilities
        # )
        # server.projects.delete_workbook_default_permissions(project, rules_to_delete)
        # server.projects.delete(project.id)


if __name__ == "__main__":
    main()



####
# This script demonstrates how to use the Tableau Server Client
# to query extract refresh tasks and run them as needed.
#
# To run the script, you must have installed Python 3.7 or later.
####

import argparse
import logging

import tableauserverclient as TSC


def handle_run(server, args):
    task = server.tasks.get_by_id(args.id)
    print(server.tasks.run(task))


def handle_list(server, _):
    tasks, pagination = server.tasks.get()
    for task in tasks:
        print("{}".format(task))


def handle_info(server, args):
    task = server.tasks.get_by_id(args.id)
    print("{}".format(task))


def main():
    parser = argparse.ArgumentParser(description="Get all of the refresh tasks available on a server")
    # Common options; please keep those in sync across all samples
    parser.add_argument("--server", "-s", required=True, help="server address")
    parser.add_argument("--site", "-S", help="site name")
    parser.add_argument(
        "--token-name", "-p", required=True, help="name of the personal access token used to sign into the server"
    )
    parser.add_argument(
        "--token-value", "-v", required=True, help="value of the personal access token used to sign into the server"
    )
    parser.add_argument(
        "--logging-level",
        "-l",
        choices=["debug", "info", "error"],
        default="error",
        help="desired logging level (set to error by default)",
    )
    # Options specific to this sample
    subcommands = parser.add_subparsers()

    list_arguments = subcommands.add_parser("list")
    list_arguments.set_defaults(func=handle_list)

    run_arguments = subcommands.add_parser("run")
    run_arguments.add_argument("id", default=None)
    run_arguments.set_defaults(func=handle_run)

    info_arguments = subcommands.add_parser("info")
    info_arguments.add_argument("id", default=None)
    info_arguments.set_defaults(func=handle_info)

    args = parser.parse_args()

    # Set logging level based on user input, or error by default
    logging_level = getattr(logging, args.logging_level.upper())
    logging.basicConfig(level=logging_level)

    # SIGN IN
    tableau_auth = TSC.PersonalAccessTokenAuth(args.token_name, args.token_value, site_id=args.site)
    server = TSC.Server(args.server, use_server_version=True)
    with server.auth.sign_in(tableau_auth):
        args.func(server, args)


if __name__ == "__main__":
    main()



import unittest

import tableauserverclient as TSC


class GroupModelTests(unittest.TestCase):
    def test_invalid_name(self):
        self.assertRaises(ValueError, TSC.GroupItem, None)
        self.assertRaises(ValueError, TSC.GroupItem, "")
        group = TSC.GroupItem("grp")
        with self.assertRaises(ValueError):
            group.name = None

        with self.assertRaises(ValueError):
            group.name = ""

    def test_invalid_minimum_site_role(self):
        group = TSC.GroupItem("grp")
        with self.assertRaises(ValueError):
            group.minimum_site_role = "Captain"

    def test_invalid_license_mode(self):
        group = TSC.GroupItem("grp")
        with self.assertRaises(ValueError):
            group.license_mode = "off"



class ResourceReference(object):
    def __init__(self, id_, tag_name):
        self.id = id_
        self.tag_name = tag_name

    def __str__(self):
        return "<ResourceReference id={} tag={}>".format(self._id, self._tag_name)

    __repr__ = __str__

    @property
    def id(self):
        return self._id

    @id.setter
    def id(self, value):
        self._id = value

    @property
    def tag_name(self):
        return self._tag_name

    @tag_name.setter
    def tag_name(self, value):
        self._tag_name = value



from defusedxml.ElementTree import fromstring


class DataAccelerationReportItem(object):
    class ComparisonRecord(object):
        def __init__(
            self,
            site,
            sheet_uri,
            unaccelerated_session_count,
            avg_non_accelerated_plt,
            accelerated_session_count,
            avg_accelerated_plt,
        ):
            self._site = site
            self._sheet_uri = sheet_uri
            self._unaccelerated_session_count = unaccelerated_session_count
            self._avg_non_accelerated_plt = avg_non_accelerated_plt
            self._accelerated_session_count = accelerated_session_count
            self._avg_accelerated_plt = avg_accelerated_plt

        @property
        def site(self):
            return self._site

        @property
        def sheet_uri(self):
            return self._sheet_uri

        @property
        def unaccelerated_session_count(self):
            return self._unaccelerated_session_count

        @property
        def accelerated_session_count(self):
            return self._accelerated_session_count

        @property
        def avg_accelerated_plt(self):
            return self._avg_accelerated_plt

        @property
        def avg_non_accelerated_plt(self):
            return self._avg_non_accelerated_plt

    def __init__(self, comparison_records):
        self._comparison_records = comparison_records

    @property
    def comparison_records(self):
        return self._comparison_records

    @staticmethod
    def _parse_element(comparison_record_xml, ns):
        site = comparison_record_xml.get("site", None)
        sheet_uri = comparison_record_xml.get("sheetURI", None)
        unaccelerated_session_count = comparison_record_xml.get("unacceleratedSessionCount", None)
        avg_non_accelerated_plt = comparison_record_xml.get("averageNonAcceleratedPLT", None)
        accelerated_session_count = comparison_record_xml.get("acceleratedSessionCount", None)
        avg_accelerated_plt = comparison_record_xml.get("averageAcceleratedPLT", None)
        return (
            site,
            sheet_uri,
            unaccelerated_session_count,
            avg_non_accelerated_plt,
            accelerated_session_count,
            avg_accelerated_plt,
        )

    @classmethod
    def from_response(cls, resp, ns):
        comparison_records = list()
        parsed_response = fromstring(resp)
        all_comparison_records_xml = parsed_response.findall(".//t:comparisonRecord", namespaces=ns)
        for comparison_record_xml in all_comparison_records_xml:
            (
                site,
                sheet_uri,
                unaccelerated_session_count,
                avg_non_accelerated_plt,
                accelerated_session_count,
                avg_accelerated_plt,
            ) = cls._parse_element(comparison_record_xml, ns)

            comparison_record = DataAccelerationReportItem.ComparisonRecord(
                site,
                sheet_uri,
                unaccelerated_session_count,
                avg_non_accelerated_plt,
                accelerated_session_count,
                avg_accelerated_plt,
            )
            comparison_records.append(comparison_record)
        return cls(comparison_records)



from typing import List, Type, TYPE_CHECKING

from defusedxml.ElementTree import fromstring

from .property_decorators import property_is_boolean
from .target import Target

if TYPE_CHECKING:
    from .target import Target


class SubscriptionItem(object):
    def __init__(self, subject: str, schedule_id: str, user_id: str, target: "Target") -> None:
        self._id = None
        self.attach_image = True
        self.attach_pdf = False
        self.message = None
        self.page_orientation = None
        self.page_size_option = None
        self.schedule_id = schedule_id
        self.send_if_view_empty = True
        self.subject = subject
        self.suspended = False
        self.target = target
        self.user_id = user_id

    def __repr__(self) -> str:
        if self.id is not None:
            return "<Subscription#{_id} subject({subject}) schedule_id({schedule_id}) user_id({user_id}) \
                target({target})".format(
                **self.__dict__
            )
        else:
            return "<Subscription subject({subject}) schedule_id({schedule_id}) user_id({user_id}) \
                target({target})".format(
                **self.__dict__
            )

    @property
    def id(self):
        return self._id

    @property
    def attach_image(self) -> bool:
        return self._attach_image

    @attach_image.setter
    @property_is_boolean
    def attach_image(self, value: bool):
        self._attach_image = value

    @property
    def attach_pdf(self) -> bool:
        return self._attach_pdf

    @attach_pdf.setter
    @property_is_boolean
    def attach_pdf(self, value: bool) -> None:
        self._attach_pdf = value

    @property
    def send_if_view_empty(self) -> bool:
        return self._send_if_view_empty

    @send_if_view_empty.setter
    @property_is_boolean
    def send_if_view_empty(self, value: bool) -> None:
        self._send_if_view_empty = value

    @property
    def suspended(self) -> bool:
        return self._suspended

    @suspended.setter
    @property_is_boolean
    def suspended(self, value: bool) -> None:
        self._suspended = value

    @classmethod
    def from_response(cls: Type, xml: bytes, ns) -> List["SubscriptionItem"]:
        parsed_response = fromstring(xml)
        all_subscriptions_xml = parsed_response.findall(".//t:subscription", namespaces=ns)

        all_subscriptions = [SubscriptionItem._parse_element(x, ns) for x in all_subscriptions_xml]
        return all_subscriptions

    @classmethod
    def _parse_element(cls, element, ns):
        schedule_element = element.find(".//t:schedule", namespaces=ns)
        content_element = element.find(".//t:content", namespaces=ns)
        user_element = element.find(".//t:user", namespaces=ns)

        # Schedule element
        schedule_id = None
        if schedule_element is not None:
            schedule_id = schedule_element.get("id", None)

        # Content element
        target = None
        send_if_view_empty = None
        if content_element is not None:
            target = Target(content_element.get("id", None), content_element.get("type"))
            send_if_view_empty = string_to_bool(content_element.get("sendIfViewEmpty", ""))

        # User element
        user_id = None
        if user_element is not None:
            user_id = user_element.get("id", None)

        # Main attributes
        id_ = element.get("id", None)
        subject = element.get("subject", None)
        attach_image = string_to_bool(element.get("attachImage", ""))
        attach_pdf = string_to_bool(element.get("attachPdf", ""))
        message = element.get("message", None)
        page_orientation = element.get("pageOrientation", None)
        page_size_option = element.get("pageSizeOption", None)
        suspended = string_to_bool(element.get("suspended", ""))

        # Create SubscriptionItem and set fields
        sub = cls(subject, schedule_id, user_id, target)
        sub._id = id_
        sub.attach_image = attach_image
        sub.attach_pdf = attach_pdf
        sub.message = message
        sub.page_orientation = page_orientation
        sub.page_size_option = page_size_option
        sub.send_if_view_empty = send_if_view_empty
        sub.suspended = suspended

        return sub


# Used to convert string represented boolean to a boolean type
def string_to_bool(s: str) -> bool:
    return s.lower() == "true"



import io
import os
import unittest
from typing import List
from unittest.mock import MagicMock

import requests_mock

import tableauserverclient as TSC
from tableauserverclient.datetime_helpers import format_datetime

TEST_ASSET_DIR = os.path.join(os.path.dirname(__file__), "assets")

GET_XML = os.path.join(TEST_ASSET_DIR, "user_get.xml")
GET_EMPTY_XML = os.path.join(TEST_ASSET_DIR, "user_get_empty.xml")
GET_BY_ID_XML = os.path.join(TEST_ASSET_DIR, "user_get_by_id.xml")
UPDATE_XML = os.path.join(TEST_ASSET_DIR, "user_update.xml")
ADD_XML = os.path.join(TEST_ASSET_DIR, "user_add.xml")
POPULATE_WORKBOOKS_XML = os.path.join(TEST_ASSET_DIR, "user_populate_workbooks.xml")
GET_FAVORITES_XML = os.path.join(TEST_ASSET_DIR, "favorites_get.xml")
POPULATE_GROUPS_XML = os.path.join(TEST_ASSET_DIR, "user_populate_groups.xml")

USERNAMES = os.path.join(TEST_ASSET_DIR, "Data", "usernames.csv")
USERS = os.path.join(TEST_ASSET_DIR, "Data", "user_details.csv")


class UserTests(unittest.TestCase):
    def setUp(self) -> None:
        self.server = TSC.Server("http://test", False)

        # Fake signin
        self.server._site_id = "dad65087-b08b-4603-af4e-2887b8aafc67"
        self.server._auth_token = "j80k54ll2lfMZ0tv97mlPvvSCRyD0DOM"

        self.baseurl = self.server.users.baseurl

    def test_get(self) -> None:
        with open(GET_XML, "rb") as f:
            response_xml = f.read().decode("utf-8")
        with requests_mock.mock() as m:
            m.get(self.baseurl + "?fields=_all_", text=response_xml)
            all_users, pagination_item = self.server.users.get()

        self.assertEqual(2, pagination_item.total_available)
        self.assertEqual(2, len(all_users))

        self.assertTrue(any(user.id == "dd2239f6-ddf1-4107-981a-4cf94e415794" for user in all_users))
        single_user = next(user for user in all_users if user.id == "dd2239f6-ddf1-4107-981a-4cf94e415794")
        self.assertEqual("alice", single_user.name)
        self.assertEqual("Publisher", single_user.site_role)
        self.assertEqual("2016-08-16T23:17:06Z", format_datetime(single_user.last_login))
        self.assertEqual("alice cook", single_user.fullname)
        self.assertEqual("alicecook@test.com", single_user.email)

        self.assertTrue(any(user.id == "2a47bbf8-8900-4ebb-b0a4-2723bd7c46c3" for user in all_users))
        single_user = next(user for user in all_users if user.id == "2a47bbf8-8900-4ebb-b0a4-2723bd7c46c3")
        self.assertEqual("Bob", single_user.name)
        self.assertEqual("Interactor", single_user.site_role)
        self.assertEqual("Bob Smith", single_user.fullname)
        self.assertEqual("bob@test.com", single_user.email)

    def test_get_empty(self) -> None:
        with open(GET_EMPTY_XML, "rb") as f:
            response_xml = f.read().decode("utf-8")
        with requests_mock.mock() as m:
            m.get(self.baseurl, text=response_xml)
            all_users, pagination_item = self.server.users.get()

        self.assertEqual(0, pagination_item.total_available)
        self.assertEqual([], all_users)

    def test_get_before_signin(self) -> None:
        self.server._auth_token = None
        self.assertRaises(TSC.NotSignedInError, self.server.users.get)

    def test_get_by_id(self) -> None:
        with open(GET_BY_ID_XML, "rb") as f:
            response_xml = f.read().decode("utf-8")
        with requests_mock.mock() as m:
            m.get(self.baseurl + "/dd2239f6-ddf1-4107-981a-4cf94e415794", text=response_xml)
            single_user = self.server.users.get_by_id("dd2239f6-ddf1-4107-981a-4cf94e415794")

        self.assertEqual("dd2239f6-ddf1-4107-981a-4cf94e415794", single_user.id)
        self.assertEqual("alice", single_user.name)
        self.assertEqual("Alice", single_user.fullname)
        self.assertEqual("Publisher", single_user.site_role)
        self.assertEqual("ServerDefault", single_user.auth_setting)
        self.assertEqual("2016-08-16T23:17:06Z", format_datetime(single_user.last_login))
        self.assertEqual("local", single_user.domain_name)

    def test_get_by_id_missing_id(self) -> None:
        self.assertRaises(ValueError, self.server.users.get_by_id, "")

    def test_update(self) -> None:
        with open(UPDATE_XML, "rb") as f:
            response_xml = f.read().decode("utf-8")
        with requests_mock.mock() as m:
            m.put(self.baseurl + "/dd2239f6-ddf1-4107-981a-4cf94e415794", text=response_xml)
            single_user = TSC.UserItem("test", "Viewer")
            single_user._id = "dd2239f6-ddf1-4107-981a-4cf94e415794"
            single_user.name = "Cassie"
            single_user.fullname = "Cassie"
            single_user.email = "cassie@email.com"
            single_user = self.server.users.update(single_user)

        self.assertEqual("Cassie", single_user.name)
        self.assertEqual("Cassie", single_user.fullname)
        self.assertEqual("cassie@email.com", single_user.email)
        self.assertEqual("Viewer", single_user.site_role)

    def test_update_missing_id(self) -> None:
        single_user = TSC.UserItem("test", "Interactor")
        self.assertRaises(TSC.MissingRequiredFieldError, self.server.users.update, single_user)

    def test_remove(self) -> None:
        with requests_mock.mock() as m:
            m.delete(self.baseurl + "/dd2239f6-ddf1-4107-981a-4cf94e415794", status_code=204)
            self.server.users.remove("dd2239f6-ddf1-4107-981a-4cf94e415794")

    def test_remove_with_replacement(self) -> None:
        with requests_mock.mock() as m:
            m.delete(
                self.baseurl
                + "/dd2239f6-ddf1-4107-981a-4cf94e415794"
                + "?mapAssetsTo=4cc4c17f-898a-4de4-abed-a1681c673ced",
                status_code=204,
            )
            self.server.users.remove("dd2239f6-ddf1-4107-981a-4cf94e415794", "4cc4c17f-898a-4de4-abed-a1681c673ced")

    def test_remove_missing_id(self) -> None:
        self.assertRaises(ValueError, self.server.users.remove, "")

    def test_add(self) -> None:
        with open(ADD_XML, "rb") as f:
            response_xml = f.read().decode("utf-8")
        with requests_mock.mock() as m:
            m.post(self.baseurl + "", text=response_xml)
            new_user = TSC.UserItem(name="Cassie", site_role="Viewer", auth_setting="ServerDefault")
            new_user = self.server.users.add(new_user)

        self.assertEqual("4cc4c17f-898a-4de4-abed-a1681c673ced", new_user.id)
        self.assertEqual("Cassie", new_user.name)
        self.assertEqual("Viewer", new_user.site_role)
        self.assertEqual("ServerDefault", new_user.auth_setting)

    def test_populate_workbooks(self) -> None:
        with open(POPULATE_WORKBOOKS_XML, "rb") as f:
            response_xml = f.read().decode("utf-8")
        with requests_mock.mock() as m:
            m.get(self.baseurl + "/dd2239f6-ddf1-4107-981a-4cf94e415794/workbooks", text=response_xml)
            single_user = TSC.UserItem("test", "Interactor")
            single_user._id = "dd2239f6-ddf1-4107-981a-4cf94e415794"
            self.server.users.populate_workbooks(single_user)

            workbook_list = list(single_user.workbooks)
            self.assertEqual("3cc6cd06-89ce-4fdc-b935-5294135d6d42", workbook_list[0].id)
            self.assertEqual("SafariSample", workbook_list[0].name)
            self.assertEqual("SafariSample", workbook_list[0].content_url)
            self.assertEqual(False, workbook_list[0].show_tabs)
            self.assertEqual(26, workbook_list[0].size)
            self.assertEqual("2016-07-26T20:34:56Z", format_datetime(workbook_list[0].created_at))
            self.assertEqual("2016-07-26T20:35:05Z", format_datetime(workbook_list[0].updated_at))
            self.assertEqual("ee8c6e70-43b6-11e6-af4f-f7b0d8e20760", workbook_list[0].project_id)
            self.assertEqual("default", workbook_list[0].project_name)
            self.assertEqual("5de011f8-5aa9-4d5b-b991-f462c8dd6bb7", workbook_list[0].owner_id)
            self.assertEqual(set(["Safari", "Sample"]), workbook_list[0].tags)

    def test_populate_workbooks_missing_id(self) -> None:
        single_user = TSC.UserItem("test", "Interactor")
        self.assertRaises(TSC.MissingRequiredFieldError, self.server.users.populate_workbooks, single_user)

    def test_populate_favorites(self) -> None:
        self.server.version = "2.5"
        baseurl = self.server.favorites.baseurl
        single_user = TSC.UserItem("test", "Interactor")
        with open(GET_FAVORITES_XML, "rb") as f:
            response_xml = f.read().decode("utf-8")
        with requests_mock.mock() as m:
            m.get("{0}/{1}".format(baseurl, single_user.id), text=response_xml)
            self.server.users.populate_favorites(single_user)
        self.assertIsNotNone(single_user._favorites)
        self.assertEqual(len(single_user.favorites["workbooks"]), 1)
        self.assertEqual(len(single_user.favorites["views"]), 1)
        self.assertEqual(len(single_user.favorites["projects"]), 1)
        self.assertEqual(len(single_user.favorites["datasources"]), 1)

        workbook = single_user.favorites["workbooks"][0]
        view = single_user.favorites["views"][0]
        datasource = single_user.favorites["datasources"][0]
        project = single_user.favorites["projects"][0]

        self.assertEqual(workbook.id, "6d13b0ca-043d-4d42-8c9d-3f3313ea3a00")
        self.assertEqual(view.id, "d79634e1-6063-4ec9-95ff-50acbf609ff5")
        self.assertEqual(datasource.id, "e76a1461-3b1d-4588-bf1b-17551a879ad9")
        self.assertEqual(project.id, "1d0304cd-3796-429f-b815-7258370b9b74")

    def test_populate_groups(self) -> None:
        self.server.version = "3.7"
        with open(POPULATE_GROUPS_XML, "rb") as f:
            response_xml = f.read().decode("utf-8")
        with requests_mock.mock() as m:
            m.get(self.server.users.baseurl + "/dd2239f6-ddf1-4107-981a-4cf94e415794/groups", text=response_xml)
            single_user = TSC.UserItem("test", "Interactor")
            single_user._id = "dd2239f6-ddf1-4107-981a-4cf94e415794"
            self.server.users.populate_groups(single_user)

            group_list = list(single_user.groups)

            self.assertEqual(3, len(group_list))
            self.assertEqual("ef8b19c0-43b6-11e6-af50-63f5805dbe3c", group_list[0].id)
            self.assertEqual("All Users", group_list[0].name)
            self.assertEqual("local", group_list[0].domain_name)

            self.assertEqual("e7833b48-c6f7-47b5-a2a7-36e7dd232758", group_list[1].id)
            self.assertEqual("Another group", group_list[1].name)
            self.assertEqual("local", group_list[1].domain_name)

            self.assertEqual("86a66d40-f289-472a-83d0-927b0f954dc8", group_list[2].id)
            self.assertEqual("TableauExample", group_list[2].name)
            self.assertEqual("local", group_list[2].domain_name)

    def test_get_usernames_from_file(self):
        with open(ADD_XML, "rb") as f:
            response_xml = f.read().decode("utf-8")
        with requests_mock.mock() as m:
            m.post(self.server.users.baseurl, text=response_xml)
            user_list, failures = self.server.users.create_from_file(USERNAMES)
        assert user_list[0].name == "Cassie", user_list
        assert failures == [], failures

    def test_get_users_from_file(self):
        with open(ADD_XML, "rb") as f:
            response_xml = f.read().decode("utf-8")
        with requests_mock.mock() as m:
            m.post(self.server.users.baseurl, text=response_xml)
            users, failures = self.server.users.create_from_file(USERS)
        assert users[0].name == "Cassie", users
        assert failures == []



import unittest

import requests_mock

import tableauserverclient as TSC
from ._utils import read_xml_asset

GET_XML = "table_get.xml"
UPDATE_XML = "table_update.xml"


class TableTests(unittest.TestCase):
    def setUp(self):
        self.server = TSC.Server("http://test", False)

        # Fake signin
        self.server._site_id = "dad65087-b08b-4603-af4e-2887b8aafc67"
        self.server._auth_token = "j80k54ll2lfMZ0tv97mlPvvSCRyD0DOM"
        self.server.version = "3.5"

        self.baseurl = self.server.tables.baseurl

    def test_get(self):
        response_xml = read_xml_asset(GET_XML)
        with requests_mock.mock() as m:
            m.get(self.baseurl, text=response_xml)
            all_tables, pagination_item = self.server.tables.get()

        self.assertEqual(4, pagination_item.total_available)
        self.assertEqual("10224773-ecee-42ac-b822-d786b0b8e4d9", all_tables[0].id)
        self.assertEqual("dim_Product", all_tables[0].name)

        self.assertEqual("53c77bc1-fb41-4342-a75a-f68ac0656d0d", all_tables[1].id)
        self.assertEqual("customer", all_tables[1].name)
        self.assertEqual("dbo", all_tables[1].schema)
        self.assertEqual("9324cf6b-ba72-4b8e-b895-ac3f28d2f0e0", all_tables[1].contact_id)
        self.assertEqual(False, all_tables[1].certified)

    def test_update(self):
        response_xml = read_xml_asset(UPDATE_XML)
        with requests_mock.mock() as m:
            m.put(self.baseurl + "/10224773-ecee-42ac-b822-d786b0b8e4d9", text=response_xml)
            single_table = TSC.TableItem("test")
            single_table._id = "10224773-ecee-42ac-b822-d786b0b8e4d9"

            single_table.contact_id = "8e1a8235-c9ee-4d61-ae82-2ffacceed8e0"
            single_table.certified = True
            single_table.certification_note = "Test"
            single_table = self.server.tables.update(single_table)

        self.assertEqual("10224773-ecee-42ac-b822-d786b0b8e4d9", single_table.id)
        self.assertEqual("8e1a8235-c9ee-4d61-ae82-2ffacceed8e0", single_table.contact_id)
        self.assertEqual(True, single_table.certified)
        self.assertEqual("Test", single_table.certification_note)

    def test_delete(self):
        with requests_mock.mock() as m:
            m.delete(self.baseurl + "/0448d2ed-590d-4fa0-b272-a2a8a24555b5", status_code=204)
            self.server.tables.delete("0448d2ed-590d-4fa0-b272-a2a8a24555b5")



import logging

from .endpoint import Endpoint, api
from .exceptions import (
    ServerResponseError,
    ServerInfoEndpointNotFoundError,
    EndpointUnavailableError,
)
from tableauserverclient.models import ServerInfoItem

logger = logging.getLogger("tableau.endpoint.server_info")


class ServerInfo(Endpoint):
    def __init__(self, server):
        self.parent_srv = server
        self._info = None

    @property
    def serverInfo(self):
        if not self._info:
            self.get()
        return self._info

    def __repr__(self):
        return "<Endpoint {}>".format(self.serverInfo)

    @property
    def baseurl(self):
        return "{0}/serverInfo".format(self.parent_srv.baseurl)

    @api(version="2.4")
    def get(self):
        """Retrieve the server info for the server.  This is an unauthenticated call"""
        try:
            server_response = self.get_unauthenticated_request(self.baseurl)
        except ServerResponseError as e:
            if e.code == "404003":
                raise ServerInfoEndpointNotFoundError(e)
            if e.code == "404001":
                raise EndpointUnavailableError(e)
            raise e

        try:
            self._info = ServerInfoItem.from_response(server_response.content, self.parent_srv.namespace)
        except Exception as e:
            logging.getLogger(self.__class__.__name__).debug(e)
            logging.getLogger(self.__class__.__name__).debug(server_response.content)
        return self._info
